#include "StdAfx.h"
#include "Callback.h"
#include <assert.h>
#include "debug.h"
#include "sensoraydevice.h"


STDMETHODIMP_(ULONG) CCallback::AddRef() { return 2; }
STDMETHODIMP_(ULONG) CCallback::Release() { return 1; }

// Fake out any COM QI'ing
//
STDMETHODIMP CCallback::QueryInterface(REFIID riid, void ** ppv)
{
    //CheckPointer(ppv,E_POINTER);
    if( ppv == NULL) 
        return E_POINTER;

    if( riid == IID_ISampleGrabberCB || riid == IID_IUnknown ) 
    {
        *ppv = (void *) static_cast<ISampleGrabberCB*> ( this );
        return NOERROR;
    }    

    return E_NOINTERFACE;
}


HRESULT STDMETHODCALLTYPE CCallback::SampleCB( 
    double SampleTime,
    IMediaSample *pSample)
{
    CString dbg;
    BYTE *pBuf;
    ULONG lsize;
    if( m_cbfunc) {
        BYTE *pBuf;
        long lsize = pSample->GetActualDataLength();
        pSample->GetPointer( &pBuf);
        if( m_cbfunc && lsize != 0) {
            (*m_cbfunc)(pBuf, lsize, m_devid, m_strmidx);
        }
        // no snapshots when using callback, just return.
        return S_OK;
    }
    if (!m_bOn)
        return S_OK;
    lsize = pSample->GetActualDataLength();
    if (lsize > MAX_LASTFRAME_SIZE)
        lsize = MAX_LASTFRAME_SIZE;
    pSample->GetPointer( &pBuf);
    if (TryEnterCriticalSection(&m_critSec)) {
        memcpy(m_last_frame, pBuf, lsize);
        m_last_size = lsize;
        SetEvent(m_hFrameEvent);
        LeaveCriticalSection(&m_critSec);
    } else {
        OutputDebugString(_T("took too long to copy the data!\n"));
    }
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CCallback::BufferCB( 
    double SampleTime,
    BYTE *pBuffer,
    long BufferLen)
{
    if (!m_bOn)
        return S_OK;
    OutputDebugString(_T("buffer\n"));
    return S_OK;
}

