/* 811 device control file 
 * Copyright Sensoray Co. 2009
 */

#include "StdAfx.h"
#include "sensoraydevice.h"
#include <assert.h>
#include "videoacc.h"
#include "debug.h"
#include "vmr9.h"
#include "Registry.h"

char AVSTREAMCAPTURENAME[256] = "Sensoray 811 BDA Analog Capture";
char AVSTREAMCROSSBARNAME[256] = "Sensoray 811 BDA Analog Xbar";

/** constructor 
*/
CSensorayDevice::CSensorayDevice(const CMid811Dll *pDLL)
{
    // device present flag starts true because MID811_ERROR_NOTINITIALIZED takes
    // precedence.
    // once initialized, device present flag is set
    m_bVMR9 = FALSE;
    m_iNumSnap = 0;
    m_colorspace = MID811_COLORSPACE_YUY2;
    m_bRAWSnapshots = TRUE;
    m_bSnapshotEnable = TRUE;
    m_pVideoRenderer = NULL;
    m_bIsRecording = FALSE;
    m_pfFileWriter = NULL;
    m_bDevicePresent = TRUE;
    m_pDLL = pDLL;
    m_p811Proxy = NULL;
    m_curFilePath[0] = '\0';
    m_pXBar                     = NULL;
    m_pAudioCaptureStreamConfig = NULL;
    m_pIVidProcAmp              = NULL;
    m_pAudioCaptureStreamConfig = NULL;
    m_pAnalogVideoDecoder = NULL;
    m_pXBar = NULL;
    m_pDump = NULL;
    m_bHasCrossbar	= FALSE;
    m_bHasAudioPin	= FALSE;
    m_bRun = FALSE;
    m_bRenderVideo	= FALSE;
    m_bRecordAudio	= TRUE;
    m_bRecordVideo	= TRUE;
    m_bMute			= TRUE;
    m_bAudioHWPresent = TRUE; // will fail later if not detected(no directsound filter)
    m_pWc7  		= NULL;
    m_pWc9			= NULL;
    m_p811Proxy = NULL;
    m_pfFileWriter = NULL;
    m_rcVid.bottom  = 0;
    m_rcClip.left   = 0;
    m_rcClip.top    = 0;
    m_rcClip.right  = 0;
    m_rcClip.bottom = 0;
    m_hwnd          = NULL;
    m_iInstance     = 0;
    m_bNC           = TRUE;
}

/** destructor 
*/
CSensorayDevice::~CSensorayDevice(void)
{
    try{
        ShutDown();
        m_pDLL = NULL;
        m_hwnd =NULL;
    } 
    catch( ... ) {
        ////OUTPUTDEBUGSTRING("destructor failed\n");
    }

}
void CSensorayDevice::ShutDown(void)
{
    if(m_bRun)
    {
        IMediaControlPtr pMC = m_pGB;
        pMC->Stop();
        m_bRun = FALSE;
        pMC = NULL;
        // release the window owner 
    }
    // release interfaces 
    ReleaseInterfaces();

    m_pAudioCaptureStreamConfig = NULL;
    m_pAnalogVideoDecoder = NULL;
    m_pXBar = NULL;
}

// initialize setting graph
HRESULT CSensorayDevice::InitDeviceControl()
{
    return S_OK;
}




void CSensorayDevice::SetPSFileName(CString FileName)
{
    if( m_pfFileWriter )
    {
        if (GetFileAttributes( FileName.GetBuffer() ) != INVALID_FILE_ATTRIBUTES)
        {
            //OUTPUTDEBUGSTRING1("deleting file %s\n", FileName.GetBuffer());
            ::DeleteFile( FileName.GetBuffer());
        }
        //Set output file name
        IFileSinkFilterPtr pFileSinkFilter = NULL;
        pFileSinkFilter = m_pfFileWriter;
        if( pFileSinkFilter )
        {
            BSTR szName = FileName.AllocSysString();
            pFileSinkFilter->SetFileName(szName,NULL);
        }
        pFileSinkFilter = NULL;
    }
}

HRESULT CSensorayDevice::SetCrossbarAudioMute(BOOL muteval)
{
    HRESULT hr = S_OK;

    m_p811Proxy = CDShowHelper::FindFilter(AVSTREAMCROSSBARNAME, m_pGB);

    if (m_p811Proxy) {
        if (FAILED(m_p811Proxy->QueryInterface(IID_IAMCrossbar, (void**)&m_pXBar))) {
            //OUTPUTDEBUGSTRING("[App]: Query IAMCrossbar Interface Failed!");
            return E_FAIL;
        }
    } else {
        return E_FAIL;
    }

 
    //Get the count of input and output pins
    if (muteval) {
        (void) m_pXBar->Route(1, -1); // desiredAudioInputIndex);
    } else {
        // WARNING hardcoded here. audio pin == 5
        (void) m_pXBar->Route(1, 5); // desiredAudioInputIndex);
    }
    //else: the desired input is the current input, do nothing
    m_pXBar = NULL;
    return hr; 
}


HRESULT CSensorayDevice::SetCrossbarByPin(int pinnum)
{
    HRESULT hr = S_OK;
    long desiredVideoInputIndex = -1;
    long desiredAudioInputIndex = -1, currentAudioInputIndex = -1;
    long cOutput = -1, cInput = -1;
    if (m_pGB == NULL)
        return 0;
    
    m_p811Proxy = CDShowHelper::FindFilter(AVSTREAMCROSSBARNAME, m_pGB);

    if(m_p811Proxy)
    {
        if(FAILED(m_p811Proxy->QueryInterface(IID_IAMCrossbar, (void**)&m_pXBar)))
        {
            //OUTPUTDEBUGSTRING("[App]: Query IAMCrossbar Interface Failed!");
            return E_FAIL;
        }
    }
    else
        return E_FAIL;

    //Get the count of input and output pins
    hr = m_pXBar->get_PinCounts(&cOutput, &cInput);
    if(FAILED(hr))
    {
        //PopUpMessage("Failed get_PinCounts.");
        m_pXBar = NULL;
        return hr;
    }
    //Find out which video input pin is desired, and its related audio input pin
    if( pinnum < cInput)
    {
        long lRelated = -1, lType = -1;
        hr = m_pXBar->get_CrossbarPinInfo(TRUE, pinnum, &lRelated, &lType);
        if(FAILED(hr))
        {
            //PopUpMessage("Failed get_CrossbarPinInfo.");
            m_pXBar = NULL;
            return hr;
        }
        desiredVideoInputIndex = pinnum;
        desiredAudioInputIndex = lRelated;
    }
    else
    {
        m_pXBar = NULL;;
        return hr;
    }

    hr = m_pXBar->CanRoute(0, desiredVideoInputIndex);
    if(hr != S_OK)
    {
        //PopUpMessage("Cannot route.");
        m_pXBar = NULL;
        return hr;		
    }
    (void)m_pXBar->Route(0, desiredVideoInputIndex);

    //else: the desired input is the current input, do nothing

    //Check if the desired audio input is already routed to audio output 
    hr = m_pXBar->get_IsRoutedTo(1, &currentAudioInputIndex);
    if(FAILED(hr))
    {
        //PopUpMessage("Failed get_IsRoutedTo.");
        m_pXBar = NULL;
        return hr;
    }

    //If the current input pin is not the one we wanted, route the wanted one to video output
    if( !m_bMute || m_bRecordAudio)
    {
        hr = m_pXBar->CanRoute(1, desiredAudioInputIndex);
        if(hr != S_OK)
        {
            //PopUpMessage("Cannot route.");
            m_pXBar = NULL;
            return hr;		
        }
        (void) m_pXBar->Route(1, desiredAudioInputIndex);
    }
    else
    {
        (void) m_pXBar->Route(1, -1);
    }

    //else: the desired input is the current input, do nothing
    m_pXBar = NULL;
    return hr; 
}

// unused
HRESULT CSensorayDevice::SetCrossbar(PhysicalConnectorType video_input)
{
    HRESULT hr = S_OK;
    long desiredVideoInputIndex = -1, currentVideoInputIndex = -1;
    long desiredAudioInputIndex = -1, currentAudioInputIndex = -1;
    long cOutput = -1, cInput = -1;

    if (m_pGB == NULL)
        return 0;

    m_p811Proxy = CDShowHelper::FindFilter(AVSTREAMCROSSBARNAME, m_pGB);

    if(m_p811Proxy)
    {
        m_pXBar = m_p811Proxy;
        if(m_pXBar == NULL) // FAILED(m_p811Proxy->QueryInterface(IID_IAMCrossbar, (void**)&m_pXBar)))
        {
            //OUTPUTDEBUGSTRING("[App]: Query IAMCrossbar Interface Failed!");
            return E_FAIL;
        }
    }
    else
        return E_FAIL;

    //Get the count of input and output pins
    hr = m_pXBar->get_PinCounts(&cOutput, &cInput);
    if(FAILED(hr))
    {
        //PopUpMessage("Failed get_PinCounts.");
        m_pXBar = NULL;
        return hr;
    }
    //Find out which video input pin is desired, and its related audio input pin
    for (long i = 0; i < cInput; i++)
    {
        long lRelated = -1, lType = -1;

        hr = m_pXBar->get_CrossbarPinInfo(TRUE, i, &lRelated, &lType);
        if(FAILED(hr))
        {
            //PopUpMessage("Failed get_CrossbarPinInfo.");
            m_pXBar = NULL;
            return hr;
        }
        if(video_input == lType)
        {
            desiredVideoInputIndex = i;
            desiredAudioInputIndex = lRelated;
            break;
        }
    }

    //Check if the desired video input is already routed to video output 
    hr = m_pXBar->get_IsRoutedTo(0, &currentVideoInputIndex);
    if(FAILED(hr))
    {
        //PopUpMessage("Failed get_IsRoutedTo.");
        m_pXBar = NULL;
        return hr;
    }
    //If the current input pin is not the one we wanted, route the wanted one to video output
    if(desiredVideoInputIndex != currentVideoInputIndex)
    {
        hr = m_pXBar->CanRoute(0, desiredVideoInputIndex);
        if(hr != S_OK)
        {
            //PopUpMessage("Cannot route.");
            m_pXBar = NULL;
            return hr;		
        }
        (void)m_pXBar->Route(0, desiredVideoInputIndex);
    }
    //else: the desired input is the current input, do nothing

    //Check if the desired audio input is already routed to audio output 
    hr = m_pXBar->get_IsRoutedTo(1, &currentAudioInputIndex);
    if(FAILED(hr))
    {
        //PopUpMessage("Failed get_IsRoutedTo.");
        m_pXBar = NULL;
        return hr;
    }
    //If the current input pin is not the one we wanted, route the wanted one to video output
    //if(desiredAudioInputIndex != currentAudioInputIndex)
    if( !m_bMute || m_bRecordAudio) {
        hr = m_pXBar->CanRoute(1, desiredAudioInputIndex);
        if(hr != S_OK)
        {
            //PopUpMessage("Cannot route.");
            m_pXBar = NULL;
            return hr;		
        }
        (void) m_pXBar->Route(1, desiredAudioInputIndex);
    } else {
        // mute the audio
        (void) m_pXBar->Route(1, -1 );
    }

    //else: the desired input is the current input, do nothing
    m_pXBar = NULL;
    return hr; 
}


// configures the TV standard
HRESULT CSensorayDevice::SetAnalogVideoDecoder(AnalogVideoStandard video_standard)
{
    HRESULT hr = S_OK;
    if (m_bRun == TRUE)
        return hr;

    if (m_pGB == NULL)
        return 0;

    m_p811Proxy = CDShowHelper::FindFilter(AVSTREAMCAPTURENAME, m_pGB);

    if (m_p811Proxy) {
        m_pAnalogVideoDecoder = m_p811Proxy;
        if(m_pAnalogVideoDecoder == NULL) {
            //OUTPUTDEBUGSTRING("[App]: Query IAMAnalogVideoDecoder Interface Failed!");
            return E_FAIL;
        }
    }
    else
    {
        return E_FAIL;
    }
    hr = m_pAnalogVideoDecoder->put_TVFormat(video_standard);
    m_pAnalogVideoDecoder = NULL;
    return hr;
}

// Sets to Vcr mode to avoid washed out video on 811
HRESULT CSensorayDevice::SetVcrMode(int mode)
{
    HRESULT hr = S_OK;
    if (m_pGB == NULL)
        return 0;
    m_p811Proxy = CDShowHelper::FindFilter(AVSTREAMCAPTURENAME, m_pGB);
    if (m_p811Proxy) {
        m_pAnalogVideoDecoder = m_p811Proxy;
        if(m_pAnalogVideoDecoder == NULL) {
            //OUTPUTDEBUGSTRING("[App]: Query IAMAnalogVideoDecoder Interface Failed!");
            return E_FAIL;
        }
    } else {
        return E_FAIL;
    }
    // 811 looks washed out unless VCR input selected.
    m_pAnalogVideoDecoder->put_VCRHorizontalLocking(mode);
    m_pAnalogVideoDecoder = NULL;
    return hr;
}

// Just calls buildsourcegraph
HRESULT CSensorayDevice::BuildPreviewGraph()
{
    HRESULT hr = S_OK;
    hr = Build811PreviewGraph();
    return hr;
}

HRESULT CSensorayDevice::PauseGraph()
{
    HRESULT hr = S_OK;
    IMediaControlPtr pMC = m_pGB;
    if (pMC == NULL)
        return -1;

    pMC->Pause();
    pMC = NULL;
    return hr;
}

HRESULT CSensorayDevice::ResumeGraph()
{
    HRESULT hr = S_OK;
    IMediaControlPtr pMC = m_pGB;
    if( pMC == NULL)
        return -1;
    pMC->Run();
    pMC = NULL;
    return hr;
}


// stops the stream from running.
void CSensorayDevice::StopGraph()
{
    if(m_bRun) {
        IMediaControlPtr pMC = m_pGB;
        pMC->Stop();
        m_bRun = FALSE;
        pMC = NULL;
        m_iNumSnap = 0;
        SetOwnerWindow(NULL);
    }
}

void CSensorayDevice::CleanUp()
{
    ReleaseInterfaces();
    m_p811Proxy = NULL;
    m_pfFileWriter = NULL;
    m_pGB = NULL;
}

// returns state of Filter Graph, running or not 
BOOL CSensorayDevice::IsGraphRunning() const
{
    return m_bRun;
}


void CSensorayDevice::QueryInterfaces()
{
    m_p811Proxy = CDShowHelper::FindFilter(AVSTREAMCAPTURENAME, m_pGB); 
    if(m_p811Proxy)
    {
        m_pIVidProcAmp = m_p811Proxy;
        if( m_pIVidProcAmp == NULL)
        {
            //OUTPUTDEBUGSTRING1("IAMVideoProcAmp %x\n",GetLastError());
        }

    }
}

void CSensorayDevice::ReleaseInterfaces()
{
    m_pIVidProcAmp = NULL;
    m_pfFileWriter = NULL;
    m_pAudioCaptureStreamConfig = NULL;
    m_pXBar = NULL;
    m_pIVidProcAmp = NULL;
    m_pAnalogVideoDecoder = NULL;
    m_pVideoRenderer = NULL;
    m_pDump = NULL;
    m_p811Proxy = NULL;
    m_pfFileWriter = NULL;
    m_pWc7 = NULL;   // pointer to windowless control	
    m_pWc9 = NULL;   // pointer to windowless control DirectX 9
    m_pISG = NULL;
}

// set input
HRESULT CSensorayDevice::SetInput(int input)
{
    // set the input channel
    HRESULT hr;
    if (m_pGB == NULL)
        return 0;
    hr = this->SetCrossbarByPin( input);
    return hr;
}


void CSensorayDevice::SetRecordVideo( BOOL bRec)
{
    m_bRecordVideo = bRec;
    return;
}
BOOL CSensorayDevice::GetRecordVideo( ) const
{
    return m_bRecordVideo;
}

void CSensorayDevice::SetRecordAudio( BOOL bRec)
{
    m_bRecordAudio = bRec;

    return;
}

BOOL CSensorayDevice::GetRecordAudio( ) const
{
    return m_bRecordAudio;
}

void CSensorayDevice::SetMute( BOOL bMute)
{
    m_bMute = bMute;
    return;
}

BOOL CSensorayDevice::GetMute() const
{
    return m_bMute;
}


void CSensorayDevice::SetRenderVideo( BOOL bRenderVid)
{
    m_bRenderVideo = bRenderVid;
    return;
}


BOOL CSensorayDevice::GetRenderVideo( ) const
{
    return m_bRenderVideo;
}

HRESULT CSensorayDevice::SetBrightness(int bright)
{
    HRESULT hr = S_OK;
    //if ( bright > 100 || bright < 0 ) return E_INVALIDARG;
    if( m_pIVidProcAmp == NULL) return -1;

    hr = m_pIVidProcAmp->Set( KSPROPERTY_VIDEOPROCAMP_BRIGHTNESS, bright, 
        KSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL);  // see ksmedia.h
    if( FAILED(hr))
    {
        //OUTPUTDEBUGSTRING1( "failed to set brightness %d", bright);
    }
    return hr;
}

HRESULT CSensorayDevice::SetContrast(int contrast)
{
    HRESULT hr;
    //if ( contrast > 100 || contrast < 0 ) return E_INVALIDARG;
    if( m_pIVidProcAmp == NULL) return -1;

    hr = m_pIVidProcAmp->Set( KSPROPERTY_VIDEOPROCAMP_CONTRAST, contrast, 
        KSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL);  // see ksmedia.h

#ifdef DEBUG
    if(FAILED(hr)) 
    {
        //OUTPUTDEBUGSTRING1("failed to set contrast: %d\n", contrast);
    }
#endif
    return hr;
}
HRESULT CSensorayDevice::SetSaturation(int sat)
{
    HRESULT hr;
    //if( sat > 100 || sat < 0 ) return E_INVALIDARG;
    if( m_pIVidProcAmp == NULL) return -1;

    hr = m_pIVidProcAmp->Set( KSPROPERTY_VIDEOPROCAMP_SATURATION, sat, 
        KSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL);  // see ksmedia.h
#ifdef DEBUG
    if( FAILED(hr))
    {
        //OUTPUTDEBUGSTRING1("failed to set saturation: %d\n", sat);
    }
#endif
    return hr;
}

HRESULT CSensorayDevice::SetHue(int hue)
{
    HRESULT hr;
    //if( hue > 50 || hue < -50 ) return E_INVALIDARG;
    if( m_pIVidProcAmp == NULL) return -1;

    hr = m_pIVidProcAmp->Set( KSPROPERTY_VIDEOPROCAMP_HUE, hue, 
        KSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL);  // see ksmedia.h
#ifdef DEBUG
    if( FAILED(hr))
    {
        //OUTPUTDEBUGSTRING1("failed to set hue: %d\n", hue);
    }
#endif
    return hr;
}


#ifdef ENABLE_SNAPSHOTS
/** returns dynamically allocated buffer as ppBuffer. 
*  The caller is responsible for freeing memory allocated with this call.
*  returns size in lSize
*  Gets the snapshot buffer from the mpeg stream
*/
HRESULT CSensorayDevice::GetSnapshotBufferNC( long *lSize, void **ppBuffer, 
                                            BITMAPINFOHEADER *pBMI )
{
    long bufSize = 0;
    HRESULT hr;

    *lSize = 0;
    if( !m_bSnapshotEnable) {
        return -1;
    }
    // get the buffer size 
    hr = m_pISG->GetCurrentBuffer( &bufSize, NULL);
    if( FAILED(hr)) {
        return hr;
    }
    if( m_pISG ) {
        unsigned char *pSample;
        pSample = (unsigned char *) malloc( bufSize);
        hr = m_pISG->GetCurrentBuffer( &bufSize, (long *)  pSample);
        if( FAILED(hr)) {
            free(pSample);
            return hr;
        }
        // sanity check. if first sample check the media type
        if (!m_iNumSnap) {
            AM_MEDIA_TYPE mt;
            hr = m_pISG->GetConnectedMediaType( &mt);
            if( FAILED(hr))
                return hr;
            if( mt.formattype != FORMAT_VideoInfo) {
                //OUTPUTDEBUGSTRING("invalid media type\n");
                FreeMediaType( mt);
                return -1;
            }
            VIDEOINFOHEADER* format = (VIDEOINFOHEADER*)mt.pbFormat;
            m_bmiSG = format->bmiHeader;
            // check format
            if( mt.subtype == MEDIASUBTYPE_YUY2) {
                //OUTPUTDEBUGSTRING( "YUY2");
            }
            else if( mt.subtype == MEDIASUBTYPE_RGB24) {
                //OUTPUTDEBUGSTRING( "RGB24");
            }
            else {
                //OUTPUTDEBUGSTRING("unknown subtype\n");
            }
            FreeMediaType( mt);
        }
        *pBMI = m_bmiSG;
        m_iNumSnap++;
        *ppBuffer = pSample;
        *lSize = bufSize;
    }
    else {
        *lSize = 0;
        return E_FAIL;
    }
    return S_OK;
}

/** For raw capture
*/
HRESULT CSensorayDevice::GetSnapshotBufferStaticNC( long *lSize, unsigned char *pBuffer,  BITMAPINFOHEADER *pBMI )
{
    HRESULT hr = S_OK;
    if( m_pISG ) {
        //OUTPUTDEBUGSTRING("using current buffer\n");
        hr = m_pISG->GetCurrentBuffer(lSize, (long *) pBuffer);
        if( FAILED(hr))
            return hr;
        // sanity check. if first sample check the media type
        if( !m_iNumSnap ) {
            AM_MEDIA_TYPE mt;
            hr = m_pISG->GetConnectedMediaType( &mt);
            if( FAILED(hr))
                return hr;
            if( mt.formattype != FORMAT_VideoInfo) {
                //OUTPUTDEBUGSTRING("invalid media type\n");
                FreeMediaType( mt);
                return -1;
            }
            VIDEOINFOHEADER* format = (VIDEOINFOHEADER*)mt.pbFormat;
            m_bmiSG = format->bmiHeader;
            // check format
            if( mt.subtype == MEDIASUBTYPE_YUY2) {
                //OUTPUTDEBUGSTRING( "YUY2");
            }
            else if( mt.subtype == MEDIASUBTYPE_RGB24) {
                //OUTPUTDEBUGSTRING( "RGB24");
            }
            else {
                //OUTPUTDEBUGSTRING("unknown subtype\n");
            }
            FreeMediaType( mt);
        }
        *pBMI = m_bmiSG;
        m_iNumSnap++;
        return S_OK;
    }
    else {
        *lSize = 0;
        return E_FAIL;
    }
}

HRESULT CSensorayDevice::GetSnapshotBufferStatic( long *lSize, unsigned char *pBuffer,int type, BITMAPINFOHEADER *pBMI )
{
    if( m_pISG) {
        return GetSnapshotBufferStaticNC( lSize, pBuffer, pBMI);
    }
    return -1;
}

#endif


HRESULT CSensorayDevice::DisplayModeChanged()
{   
    HRESULT hr = -1;
    if( m_bVMR9) {
        if( m_pWc9 != NULL) {
            hr = m_pWc9->DisplayModeChanged();
        }
    }
    else {
        if( m_pWc7 != NULL) {
            hr = m_pWc7->DisplayModeChanged();
        }
    }
    return hr;
}

HRESULT CSensorayDevice::RepaintVideo(HDC hdc)
{
    HRESULT hr = -1;
    if( m_bVMR9) {
        if( m_pWc9 != NULL) {
            hr = m_pWc9->RepaintVideo( m_hwnd, hdc);
        }
    }
    else {
        if( m_pWc7 != NULL) {
            hr = m_pWc7->RepaintVideo( m_hwnd, hdc);
        }
    }
    return hr;
}

HRESULT CSensorayDevice::SetAspectRatioMode(MID811_ASPECT_MODE mode)
{
    HRESULT hr = -1;

    if( m_bVMR9) {
        if( m_pWc9 != NULL) {
            hr = m_pWc9->SetAspectRatioMode( mode);
        }
    }
    else {
        if( m_pWc7 != NULL) {
            hr = m_pWc7->SetAspectRatioMode( mode);
        }
    }
    return hr;
}




HRESULT CSensorayDevice::ConfigOldRenderer( IBaseFilterPtr VideoRenderer)
{
    HRESULT hr;
    IVideoWindowPtr pConfig; 
    hr = VideoRenderer->QueryInterface(IID_IVideoWindow, (void**)&pConfig); 
    if( !FAILED(hr))
    {
        hr = pConfig->put_Owner((OAHWND) m_hwnd);
        pConfig = NULL;
    }
    return hr;
}

HRESULT CSensorayDevice::ConfigVMR9( IBaseFilterPtr VideoRenderer)
{
    HRESULT hr = S_OK;
    IVMRFilterConfig9Ptr pConfig;
    pConfig = VideoRenderer; 
    if( pConfig != NULL)
    {
        hr = pConfig->SetRenderingMode( VMRMode_Windowless);
        pConfig = NULL;
        if( SUCCEEDED(hr))
        {
            m_pWc9 = VideoRenderer; 
            if( m_pWc9)
            {
                m_pWc9->SetVideoClippingWindow( m_hwnd);
            }
            // 6/22/06( needs to be set for blending bitmap for MD regions)
            IVMRFilterConfig9Ptr pCfg;
            pCfg = VideoRenderer; // ->QueryInterface( IID_IVMRFilterConfig9, (void **) &pCfg );
            if( pCfg)
            {
                pCfg->SetNumberOfStreams(1); //m_pWc->SetVideoClippingWindow( m_hwnd);
            }
            pCfg = NULL;
            //            m_pMon9 = VideoRenderer;
        }
    }
    else hr = E_FAIL;
    return hr;
}

HRESULT CSensorayDevice::ConfigVMR7( IBaseFilterPtr VideoRenderer)
{
    HRESULT hr = S_OK;
    IVMRFilterConfigPtr pConfig; 
    pConfig = VideoRenderer; // ->QueryInterface(IID_IVMRFilterConfig, (void**)&pConfig); 
    if( pConfig)
    {
        hr = pConfig->SetRenderingMode( VMRMode_Windowless);
        pConfig = NULL;
        if( SUCCEEDED(hr))
        {
            m_pWc7 = VideoRenderer; // ->QueryInterface( IID_IVMRWindowlessControl, (void **) &m_pWc );
            if( m_pWc7)
            {
                m_pWc7->SetVideoClippingWindow( m_hwnd);
            }
            // 6/22/06( needs to be set for blending bitmap for MD regions)
            IVMRFilterConfigPtr pCfg;
            pCfg = VideoRenderer; // ->QueryInterface( IID_IVMRFilterConfig, (void **) &pCfg );
            if( pCfg)
            {
                pCfg->SetNumberOfStreams(1); //m_pWc->SetVideoClippingWindow( m_hwnd);
            }
            pCfg = NULL;
            //            m_pMon = VideoRenderer;
        }
    }
    else hr = E_FAIL;
    return hr;
}


HRESULT CSensorayDevice::GetNativeVidSize( RECT *rect)
{
    HRESULT hr = -1;
    LONG lWidth, lHeight;

    if( this->m_bVMR9)
    {
        if( m_pWc9 == NULL)
            return -1;
        hr = m_pWc9->GetNativeVideoSize( &lWidth, &lHeight, NULL,NULL);
        if( FAILED(hr ))
            return hr;
    }
    else
    {
        if( m_pWc7 == NULL)
            return -1;

        hr = m_pWc7->GetNativeVideoSize( &lWidth, &lHeight, NULL,NULL);
        if( FAILED(hr ))
            return hr;
    }
    rect->left = 0;
    rect->top = 0;
    rect->bottom = lHeight;
    rect->right = lWidth;
    return S_OK;
}

/** Updates the owner window hander
*/
HRESULT CSensorayDevice::SetOwnerWindow( HWND hwnd )
{
   HRESULT hr = S_OK;
    LONG lWidth, lHeight;
    RECT  rcDst = {0,0,0,0};
    RECT *prcSrc;
    RECT  rcSrc;

    if( m_rcVid.bottom == 0) {
        if( !GetClientRect( hwnd, &rcDst)) {
            hr = GetLastError();  //lint !e713
        }
    }
    else {
        rcDst = m_rcVid;
    }

    if( hwnd != NULL) {
        m_hwnd = hwnd;
    }
#ifdef TEST_CLIP
        m_rcClip.left   =  2;
        m_rcClip.top    =  1;
        m_rcClip.right  =  7;
        m_rcClip.bottom =  7;
#endif
#ifdef TEST_SCALING
        rcDst.bottom =lHeight;
        rcDst.left   = 0;
        rcDst.top    = 0;
        rcDst.right  = lWidth;
#endif

    if( m_bVMR9)
    {
        if (m_pWc9)
        {
            (void) m_pWc9->SetVideoClippingWindow( hwnd);
            hr = m_pWc9->GetNativeVideoSize( &lWidth, &lHeight, NULL,NULL);

            if ( ((m_rcClip.left  == 0) && (m_rcClip.top    == 0) &&
                (m_rcClip.right == 0) && (m_rcClip.bottom == 0) ) || FAILED(hr))
            {
                prcSrc = NULL;  // Use all of source
            } 
            else 
            {
                rcSrc.left   = 0       + m_rcClip.left   ;
                rcSrc.top    = 0       + m_rcClip.top    ;
                rcSrc.right  = lWidth  - m_rcClip.right  ;
                rcSrc.bottom = lHeight - m_rcClip.bottom ;
                prcSrc = &rcSrc;
            }
            hr = m_pWc9->SetVideoPosition( prcSrc ,&rcDst);
        }
    }
    else
    {
        if (m_pWc7)
        {
            (void) m_pWc7->SetVideoClippingWindow( hwnd);
            hr = m_pWc7->GetNativeVideoSize( &lWidth, &lHeight, NULL,NULL);

            if ( ((m_rcClip.left  == 0) && (m_rcClip.top    == 0) &&
                (m_rcClip.right == 0) && (m_rcClip.bottom == 0) ) || FAILED(hr))
            {
                prcSrc = NULL;  // Use all of source
            } 
            else 
            {
                rcSrc.left   = 0       + m_rcClip.left   ;
                rcSrc.top    = 0       + m_rcClip.top    ;
                rcSrc.right  = lWidth  - m_rcClip.right  ;
                rcSrc.bottom = lHeight - m_rcClip.bottom ;
                prcSrc = &rcSrc;
            }
            hr = m_pWc7->SetVideoPosition( prcSrc ,&rcDst);
        }

    }
    return hr;
}
/** Sets non-standard video position in the window
*/
HRESULT CSensorayDevice::SetVideoPosition( RECT const rcVid, RECT const rcClip )
{
    HRESULT hr = S_OK;
    long  lWidth, lHeight;
    RECT *prcSrc;
    RECT  rcDst;
    RECT  rcSrc;

    m_rcVid  = rcVid;
    m_rcClip = rcClip;
    if( m_bVMR9)
    {
        if (m_pWc9)
        {
            m_pWc9->GetNativeVideoSize( &lWidth, &lHeight, NULL,NULL);
            if( m_rcVid.bottom == 0) 
            {
                if( !GetClientRect( m_hwnd, &rcDst))
                {
                    hr = GetLastError();
                    CHECK_ERROR( _T("SetVideoPosition: GetClientRect %x\n"), hr);
                }
            } 
            else 
            {
                rcDst = m_rcVid;
            }

            rcSrc.left   = 0       + m_rcClip.left   ;
            rcSrc.top    = 0       + m_rcClip.top    ;
            rcSrc.right  = lWidth  - m_rcClip.right  ;
            rcSrc.bottom = lHeight - m_rcClip.bottom ;
            prcSrc = &rcSrc;

            hr = m_pWc9->SetVideoPosition( prcSrc ,&rcDst);
        }
    }
    else
    {

        if (m_pWc7)
        {
            m_pWc7->GetNativeVideoSize( &lWidth, &lHeight, NULL,NULL);
            if( m_rcVid.bottom == 0) 
            {
                if( !GetClientRect( m_hwnd, &rcDst))
                {
                    hr = GetLastError();
                    CHECK_ERROR( _T("SetVideoPosition: GetClientRect %x\n"), hr);
                }
            } 
            else 
            {
                rcDst = m_rcVid;
            }

            rcSrc.left   = 0       + m_rcClip.left   ;
            rcSrc.top    = 0       + m_rcClip.top    ;
            rcSrc.right  = lWidth  - m_rcClip.right  ;
            rcSrc.bottom = lHeight - m_rcClip.bottom ;
            prcSrc = &rcSrc;

            hr = m_pWc7->SetVideoPosition( prcSrc ,&rcDst);
        }
    }
    return hr;
}

// Note: This Graph uses DirectShow SampleGrabber with Buffering.
// Use this graph for OCCASIONAL snapshots.  
// mode == 0 MEDIATYPE_Video, MEDIASUBTYPE_RGB24
// mode == 1 MEDIATYPE_Video, MEDIASUBTYPE_YUY2

HRESULT CSensorayDevice::BuildCaptureOnlySG(int mode)
{
    CAutoLock lock(&m_critSec);
    HRESULT hr = S_OK;
    BOOL bDeviceFound = FALSE;
    IPinPtr pPin1 = NULL;
    IPinPtr pPin2 = NULL;
    IBaseFilterPtr AVCapture;
    IBaseFilterPtr SnapshotFilter;
    IBaseFilterPtr NullRenderer;

    ::CoInitialize(NULL);
    ReleaseInterfaces();

    m_pGB = NULL;
    hr = m_pGB.CreateInstance( CLSID_FilterGraph);
    CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);


    hr = CDShowHelper::AddCategoryFilter(AM_KSCATEGORY_CAPTURE,AVSTREAMCAPTURENAME,m_pGB, &AVCapture, m_iInstance);
    CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
    bDeviceFound = TRUE;
    pPin2 = CDShowHelper::FindPin(TRUE, AVCapture);
    //If we find the analog video input pin in AVCapture filter
    if( pPin2 ) { 
        //Add Crossbar filter and connect it with AVCapture filter
		hr = AddCrossbar(AVCapture);
        CHECK_ERROR(TEXT("CSensorayDevice::"), hr);
    } else return -1;
    SetInput(m_pDLL->m_param[m_iInstance].m_input);
    SetCrossbarAudioMute(TRUE);
    // call update compression settings.
    // For NC 811, this just updates the capture pin to the correct resolution
    UpdateVideoSettings(FALSE, TRUE);
    {
        AM_MEDIA_TYPE mt;
        memset( &mt, 0, sizeof(AM_MEDIA_TYPE));
        mt.majortype = MEDIATYPE_Video;
        mt.formattype = GUID_NULL;
        m_bRAWSnapshots = TRUE;
        ADDFILTER_NEW( CLSID_DEFAULT_SG, "SRAY Video Decoder Snapshot", SnapshotFilter); 
        CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr); 
        m_pISG = SnapshotFilter;
        // make sure buffering turned on 
        m_pISG->SetBufferSamples( TRUE);
        switch( mode) {
                case 1:
                    mt.subtype = MEDIASUBTYPE_YUY2;
                    break;
                case 0:
                    mt.subtype = MEDIASUBTYPE_RGB24;
                    break;                
                default:
                    mt.subtype = MEDIASUBTYPE_YUY2;
                    break;
        }
        m_pISG->SetMediaType( &mt);
        m_bRAWSnapshots = TRUE;
    }

    CONNECT_NEW( CDShowHelper::FindPin(RAW_CAPTURE_PINNAME, AVCapture),
        CDShowHelper::FindPin(TRUE, SnapshotFilter), "cap to snapshot");
    CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
    ADDFILTER_NEW( "{C1F400A4-3F08-11D3-9F0B-006008039E37}", "Null Renderer", NullRenderer);
    CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
    m_bRAWSnapshots = TRUE;
    m_bSnapshotEnable = TRUE;
    m_bDevicePresent = bDeviceFound;
    // get the interfaces
    QueryInterfaces();
    
    return hr;
}



/** starts running the graph.  Uses appropriate references clock based on PARAM_CONFIG_FILENAME 
 *  and current video encode type.
 */
HRESULT CSensorayDevice::StartGraph()
{
    HRESULT hr;
    int iCS = GetClockSource();
    int nRefClock = 0; // default clock
    CString tmp;

    // default ref clock
    if( m_pDLL->m_param[m_iInstance].m_bLOWLatency ) {
        //OUTPUTDEBUGSTRING("2250: no ref clock for low latency\n");
        nRefClock = -1;
    }
    tmp.Format(_T("[811]: refclock: %d\n"), nRefClock);
    OUTPUTDEBUGSTRING( tmp);
    IMediaControlPtr pMC = m_pGB;
    hr = CDShowHelper::SetSyncClockGB( m_pGB, nRefClock);
    if( hr == CLASS_E_CLASSNOTAVAILABLE)
    {
        // probably does not have an audio renderer, so use the system clock
        //OUTPUTDEBUGSTRING("No audio renderer, using default clock");
        nRefClock = 0;
        hr =  CDShowHelper::SetSyncClockGB( m_pGB, nRefClock);
    }
    // make sure 811 running vcr mode or input may be washed out
    SetVcrMode(1);
    hr = pMC->Run();
    if (!FAILED(hr))
        hr = 0;
    pMC = NULL; 
    //OUTPUTDEBUGSTRING1("running graph ref clock %d\n", nRefClock);
    m_bRun = TRUE;
    return hr;
}


/** Simple interface to set the default configuration settings.
* @return 0 on success
* sets video system, Video size
*/
HRESULT CSensorayDevice::UpdateVideoSettings(BOOL bPreview, BOOL bRecord )
{
    CString strcfgName, fpscfgName, rescfgName, ctlcfgName, tempStr;
    CString strCurCompressMode, strCurMPEG4Mode, strCurSequence, strCurAspectRatio, strCurSearchRange;
    CString strCurTVStandard, strCurFrameRate;
    int iCurWidth  = 720;
    int iCurHeight = 480;
    MID811_RES eVidSize;
    MID811_VIDSYS      vsys;
    int recmode = m_pDLL->m_param[m_iInstance].m_recmode;
    CString setting;
    vsys =  m_pDLL->m_param[m_iInstance].m_vidsys;
    eVidSize = (MID811_RES)  m_pDLL->m_param[m_iInstance].m_iVidSize;
    if (m_pGB == NULL)
        return 0;
    // set tv standard
    if( vsys == MID811_VIDSYS_NTSC) {
        switch( eVidSize )
        {
        case MID811_RES_D1:
            iCurWidth = 720;
            iCurHeight = 480;
            break;
        case MID811_RES_SIF:
            iCurWidth = 352;
            iCurHeight = 240;
            break;
        case MID811_RES_CIF:
            iCurWidth = 352;
            iCurHeight = 288;
            break;
        case MID811_RES_VGA:
            iCurWidth = 640;
            iCurHeight = 480;
            break;
        case MID811_RES_4SIF:
            iCurWidth = 704;
            iCurHeight = 480;
            break;
        case MID811_RES_4CIF:
            iCurWidth = 704;
            iCurHeight = 576;
            break;
        case MID811_RES_D5:
            iCurWidth = 480;
            iCurHeight = 352;
            break;
        case MID811_RES_QQVGA:
            iCurWidth = 160;
            iCurHeight = 120;
            break;
        case MID811_RES_2SIF:
            iCurWidth = 704;
            iCurHeight = 240;
            break;
        case MID811_RES_QCIF:
            iCurWidth = 176;
            iCurHeight = 144;
            break;
        case MID811_RES_QVGA:
            iCurWidth = 320;
            iCurHeight = 240;
            break;
        case MID811_RES_D1N:
            iCurWidth = 720;
            iCurHeight = 480;
            break;
        case MID811_RES_D1P:
            iCurWidth = 720;
            iCurHeight = 576;
            break;
        case MID811_RES_SQCIF:
            iCurWidth = 128;
            iCurHeight = 96;
            break;
        }
    } else {
        switch( eVidSize )
        {
        case MID811_RES_D1:
            iCurWidth = 720;
            iCurHeight = 576;
            break;
        case MID811_RES_SIF:
            iCurWidth = 352;
            iCurHeight = 240;
            break;
        case MID811_RES_CIF:
            iCurWidth = 352;
            iCurHeight = 288;
            break;
        case MID811_RES_VGA:
            iCurWidth = 640;
            iCurHeight = 480;
            break;
        case MID811_RES_4SIF:
            iCurWidth = 704;
            iCurHeight = 480;
            break;
        case MID811_RES_4CIF:
            iCurWidth = 704;
            iCurHeight = 576;
            break;
        case MID811_RES_D5:
            iCurWidth = 480;
            iCurHeight = 352;
            break;
        case MID811_RES_QQVGA:
            iCurWidth = 160;
            iCurHeight = 112;
            break;
        case MID811_RES_2SIF:
            iCurWidth = 704;
            iCurHeight = 240;
            break;
        case MID811_RES_QCIF:
            iCurWidth = 176;
            iCurHeight = 144;
            break;
        case MID811_RES_QVGA:
            iCurWidth = 320;
            iCurHeight = 240;
            break;
        case MID811_RES_D1N:
            iCurWidth = 720;
            iCurHeight = 480;
            break;
        case MID811_RES_D1P:
            iCurWidth = 720;
            iCurHeight = 576;
            break;
        case MID811_RES_SQCIF:
            iCurWidth = 128;
            iCurHeight = 96;
            break;
        }
    } 
    // set the video system
    if( vsys == MID811_VIDSYS_NTSC) {
        
        if( FAILED(SetAnalogVideoDecoder(AnalogVideo_NTSC_M)))
        {
            OUTPUTDEBUGSTRING(_T("UpdateVideoSettings: Failed set decoder vsys\n"));
        }
    }
    else 
    {
        if( FAILED( SetAnalogVideoDecoder(AnalogVideo_PAL_B)))
        {
            OUTPUTDEBUGSTRING(_T("UpdateVideoSettings: Failed set decoder vsys\n"));
        }
    }
    // configure the video pin
    (void) SetVideoPin( iCurWidth, iCurHeight);
    return 0;
}


HRESULT CSensorayDevice::SetVideoPin(int width, int height)
{
	IAMStreamConfigPtr pVideoCaptureStreamConfig = NULL;
	HRESULT hr = S_OK;
	if(m_bRun == TRUE) {
		return hr;
    }
	m_p811Proxy = CDShowHelper::FindFilter(AVSTREAMCAPTURENAME, m_pGB);
    if(m_p811Proxy) {
        IPinPtr pPin;
        pPin = CDShowHelper::FindPin( RAW_CAPTURE_PINNAME, m_p811Proxy);
        pVideoCaptureStreamConfig = pPin;
		if(pVideoCaptureStreamConfig == NULL) {
			//OUTPUTDEBUGSTRING("[App]: Query IAMStreamConfig Interface on Capture Pin Failed!");
			hr = E_FAIL;
		}
		else {
			AM_MEDIA_TYPE* pmt;
			VIDEO_STREAM_CONFIG_CAPS caps;

			int caps_count = 0, caps_size = 0;
			pVideoCaptureStreamConfig->GetNumberOfCapabilities(&caps_count, &caps_size);

			for (int i = 0 ; i < caps_count ; i ++ )
			{
 				HRESULT hrloc = pVideoCaptureStreamConfig->GetStreamCaps(i, &pmt, (BYTE*)&caps);
				if ( FAILED(hrloc) ) { //OUTPUTDEBUGSTRING("[App]: GetSteamCaps Failed!"); continue; 
                    continue;
                }

                if ( pmt->formattype != FORMAT_VideoInfo ) goto next_stream_caps;

                VIDEOINFOHEADER* format = (VIDEOINFOHEADER*)pmt->pbFormat;
                if( m_colorspace == MID811_COLORSPACE_YUY2) {
                    if( pmt->subtype != MEDIASUBTYPE_YUY2) goto next_stream_caps;
                }
                else if( m_colorspace == MID811_COLORSPACE_RGB32) {
                    if( pmt->subtype != MEDIASUBTYPE_RGB32) goto next_stream_caps;

                }
                else if( m_colorspace == MID811_COLORSPACE_RGB24) {
                    if( pmt->subtype != MEDIASUBTYPE_RGB24) goto next_stream_caps;
                }
                else if( m_colorspace == MID811_COLORSPACE_UYVY) {
                    if( pmt->subtype != MEDIASUBTYPE_UYVY) goto next_stream_caps;
                }

                format->rcSource.left = 0;
                format->rcSource.right = width;
                format->rcSource.top = 0;
                format->rcSource.bottom = height;
                format->rcTarget.right = width;
                format->rcTarget.left = 0;
                format->rcTarget.bottom = height;
                format->rcTarget.top = 0;
                format->bmiHeader.biWidth = width;
                format->bmiHeader.biHeight = height;
                format->bmiHeader.biSizeImage = width*height * format->bmiHeader.biBitCount / 8;
				hrloc = pVideoCaptureStreamConfig->SetFormat(pmt);
                if( FAILED(hrloc) ) { 
					//OUTPUTDEBUGSTRING("[App]: set video config failed");
				}
			next_stream_caps:
					DeleteMediaType(pmt);
			}
		}
	}
	else
    {
		hr = E_FAIL;
    }
	pVideoCaptureStreamConfig = NULL;
    return hr;
}


HRESULT CSensorayDevice::GetSnapshotBuffer( long *lSize, void **ppBuffer, 
                                           BITMAPINFOHEADER *pBMI )
{
    if( m_pISG) {
        return GetSnapshotBufferNC( lSize, ppBuffer, pBMI);
    }
    return -1;
}

/** builds the record graph for NC
 * For 811NC product(no compression).
 * graph flow 811 Preview-> renderer ( no snapshots)
 *
 * with preview                   Capture->Tee->Renderer
 *                                          ->Record
 * with snapshots  Capture->SG->Record graph
 *                  
 *  with snapshots + preview) Capture->Tee->Renderer
 *                                               ->SG->Record Graph
 *                 
 * Record portion (muxer->file writer)
 */

HRESULT CSensorayDevice::BuildRecordGraph(const wchar_t *filename)
{
    CAutoLock lock(&m_critSec);
    HRESULT hr = S_OK;
    BOOL bDeviceFound = FALSE;  
    int testflags = GetTestFlags();
    int iSH = GetSnapshotMethod();
    int iAdvanced = GetAdvanced();
    BOOL bAddAudioNull = FALSE;  
    BOOL bUseDecomp = GetAviDecomp();;
    int iRenderer = GetRenderer(); 
    int recmode = m_pDLL->m_param[m_iInstance].m_recmode;
    IPinPtr pPin1 = NULL;
    IPinPtr pPin2 = NULL;
    IBaseFilterPtr AVCapture;
    IBaseFilterPtr AVEncoder;
    IBaseFilterPtr AVIDecomp;
    IBaseFilterPtr AudioTee;
    IBaseFilterPtr VideoTee = NULL;
    IBaseFilterPtr SmartTee;
    IBaseFilterPtr WaveOut;
    IBaseFilterPtr ACMWrapper;
    IBaseFilterPtr VideoRenderer;
    IBaseFilterPtr NullRenderer;
    IBaseFilterPtr NullRenderer2;
    IBaseFilterPtr Muxer;
    IBaseFilterPtr FileWriter;
    IBaseFilterPtr FileWriter2;
    IBaseFilterPtr WaveDest;
    IBaseFilterPtr VideoDec;
    IBaseFilterPtr SnapshotFilter;
    CString bstrFile;
    bstrFile = filename;
    ::CoInitialize(NULL);
    ReleaseInterfaces();
    m_pGB = NULL;
    m_colorspace = MID811_COLORSPACE_YUY2;
    hr = m_pGB.CreateInstance( CLSID_FilterGraph);

    CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
    CDShowHelper::AddCategoryFilter(AM_KSCATEGORY_CAPTURE,AVSTREAMCAPTURENAME, m_pGB, &AVCapture, m_iInstance);
    if (!(m_bRecordAudio || m_bRecordVideo)) {
        // no recording, use preview build for preview
        return -1;
    }
    bDeviceFound = TRUE;
    //If we find the analog video input pin in AVCapture filter
    if (CDShowHelper::FindPin(TRUE, AVCapture)) {
        //Add Crossbar filter and connect it with AVCapture filter
		hr = AddCrossbar(AVCapture);
        CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);

    }
    SetInput(m_pDLL->m_param[m_iInstance].m_input);
    if( !m_bMute || m_bRecordAudio) { 
        SetCrossbarAudioMute(FALSE);
    }
    else {
        SetCrossbarAudioMute(TRUE);
    }

    UpdateVideoSettings(m_bRenderVideo, m_bRecordVideo);
    if (m_bRenderVideo) {
        IPinPtr pRenderFrom;
        // default XP renderer
        //OUTPUTDEBUGSTRING("before default renderer XP\n");
        if( !iRenderer) {
            // VMR 9 default
            ADDFILTER_NEW( "{51b4abf3-748f-4e3b-a276-c828330e926a}", "VMR 9", VideoRenderer);
            // set hrBuild = -1 to debug DirectX 9( make sure to test motion detection)
            if(FAILED(hr))
            {
                //OUTPUTDEBUGSTRING("trying VMR7\n");
                hr = 0;
                ADDFILTER_NEW( "{6BC1CFFA-8FC1-4261-AC22-CFB4CC38DB50}", "Default Video Renderer", VideoRenderer);    
                if( FAILED(hr)) {
                    AfxMessageBox(_T("DirectX 9 not installed or video card not capable"));
                    return hr;
                }
                m_bVMR9 = FALSE;
                //OUTPUTDEBUGSTRING("added VMR7\n");
                ConfigVMR7( VideoRenderer);
                m_pVideoRenderer = VideoRenderer;
            }
            else
            {
                m_bVMR9 = TRUE;
                //OUTPUTDEBUGSTRING("added VMR9\n");
                hr = 0;
                assert( VideoRenderer != NULL);
                ConfigVMR9( VideoRenderer);
                m_pVideoRenderer = VideoRenderer;
            }
        }
        else {
            m_bVMR9 = FALSE;
            // choose renderer explicitly
            switch(iRenderer)
            {
            case RENDERER_VMR9:
                ADDFILTER_NEW( "{51b4abf3-748f-4e3b-a276-c828330e926a}", "VMR 9", VideoRenderer);
                CHECK_ERROR(_T("CSensorayDevice:BSG\n"), hr);
                ConfigVMR9(VideoRenderer);
                m_bVMR9 = TRUE;
                m_pVideoRenderer = VideoRenderer;
                break;
            case RENDERER_VMR7:
                ADDFILTER_NEW( "{6BC1CFFA-8FC1-4261-AC22-CFB4CC38DB50}", "VMR 7", VideoRenderer);
                CHECK_ERROR(_T("CSensorayDevice:BSG\n"), hr);
                ConfigVMR7(VideoRenderer);
                m_pVideoRenderer = VideoRenderer;
                break;
            case RENDERER_OLD:
                ADDFILTER_NEW("{70E102B0-5556-11CE-97C0-00AA0055595A}", "OldRender", VideoRenderer);
                CHECK_ERROR(_T("CSensorayDevice:BSG\n"), hr);
                ConfigOldRenderer(VideoRenderer);
                m_pVideoRenderer = VideoRenderer;
                break;
            default:
                ADDFILTER_NEW( "{6BC1CFFA-8FC1-4261-AC22-CFB4CC38DB50}", "Default Video Renderer", VideoRenderer);
                break;
            }
        }
        if( VideoRenderer == NULL) {
            return -1;
        }
        // connect to Tee Filter
        ADDFILTER_NEW("{F8388A40-D5BB-11D0-BE5A-0080C706568E}","Infinite Pin Tee For Video", VideoTee);//Infinite Tee
        CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
        CONNECT_NEW( CDShowHelper::FindPin(RAW_CAPTURE_PINNAME, AVCapture), CDShowHelper::FindPin(TRUE,VideoTee), "cap to tee");
        CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
        pRenderFrom = CDShowHelper::FirstDisconnectedPin( FALSE, VideoTee);
        
        if( bUseDecomp) {
            //OUTPUTDEBUGSTRING("SRAY811: AVI decompressor\n");
            ADDFILTER_NEW( AVI_DECOMP_CLSID , "AVI Decompressor", AVIDecomp);
            CHECK_ERROR(_T("CSensorayDevice:BSG\n"), hr);
            CONNECT_NEW( pRenderFrom, CDShowHelper::FindPin(TRUE, AVIDecomp), "preview to decomp");
            CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
            CONNECT_NEW( CDShowHelper::FindPin(FALSE, AVIDecomp), CDShowHelper::FindPin(TRUE, VideoRenderer), "decomp to render");
            CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
        }
        else {
            CONNECT_NEW( pRenderFrom, CDShowHelper::FindPin(TRUE, VideoRenderer), "capture to render");
            CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
        }
    }

    if (m_bRecordAudio || !m_bMute) {
        ADDFILTER_NEW("{F8388A40-D5BB-11D0-BE5A-0080C706568E}","Infinite Pin Tee For Audio", AudioTee);//Infinite Tee
        CHECK_ERROR(_T("CSensorayDevice:BSG\n"), hr);
        CONNECT_NEW(
            CDShowHelper::FindPin(AUDIO_PINNAME, AVCapture),
            CDShowHelper::FirstDisconnectedPin( TRUE, AudioTee),
            "Connecting Audio capture with Infinite Pin Tee"
            );
        CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
        if( !m_bMute) {
            hr = CDShowHelper::AddCategoryFilter(CLSID_AudioRendererCategory, "Default DirectSound Device", m_pGB, &WaveOut,0);
            if(FAILED(hr)) {
                m_bAudioHWPresent = FALSE; 
            }
            else{
                CONNECT_NEW(CDShowHelper::FirstDisconnectedPin( FALSE, AudioTee),CDShowHelper::FindPin(TRUE, WaveOut), "Render Audio");
                if( FAILED(hr)) {
                    m_bAudioHWPresent = FALSE;
                }
            }
        }
    } else {
        bAddAudioNull = FALSE;
    }

    ADDFILTER_NEW( CLSID_MUXER_MICROSOFT, "Muxer", Muxer);
    CHECK_ERROR(_T("CSensorayDevice:BSG\n"), hr);
    ADDFILTER_NEW( CLSID_MICROSOFT_WRITER, "FileWriter", FileWriter);
    CHECK_ERROR(_T("CSensorayDevice:BSG\n"), hr);
    m_pfFileWriter = FileWriter;
    SetPSFileName( bstrFile);
    // save the filename
    wcsncpy( m_curFilePath, filename, MAX_PATH);
    if( m_bRecordVideo) {
        if( m_bSnapshotEnable) {
            AM_MEDIA_TYPE mt;
            memset( &mt, 0, sizeof(AM_MEDIA_TYPE));
            mt.majortype = MEDIATYPE_Video;
            mt.formattype = GUID_NULL;
            m_bRAWSnapshots = TRUE;
            ADDFILTER_NEW( CLSID_DEFAULT_SG, "SRAY Video Decoder Snapshot", SnapshotFilter); 
            CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
            m_pISG = SnapshotFilter;
            // make sure buffering turned on 
            m_pISG->SetBufferSamples( TRUE);
            switch( m_colorspace) {
                default:
                case MID811_COLORSPACE_YUY2:
                    mt.subtype = MEDIASUBTYPE_YUY2;
                    break;
                case MID811_COLORSPACE_RGB32:
                    mt.subtype = MEDIASUBTYPE_RGB32;
                    break;                
                case MID811_COLORSPACE_RGB24:
                    mt.subtype = MEDIASUBTYPE_RGB24;
                    break;
                case MID811_COLORSPACE_UYVY:
                    mt.subtype = MEDIASUBTYPE_UYVY;
                    break;
            }
            m_pISG->SetMediaType( &mt);
            m_bRAWSnapshots = TRUE;
            if( VideoTee ) {
                CONNECT_NEW( CDShowHelper::FirstDisconnectedPin(FALSE, VideoTee),  CDShowHelper::FindPin(TRUE, SnapshotFilter), "capture to snapshot\n");
                CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
            }
            else {
                CONNECT_NEW( CDShowHelper::FindPin(RAW_CAPTURE_PINNAME, AVCapture),  CDShowHelper::FindPin(TRUE, SnapshotFilter), "capture to snapshot\n");
                CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
            }
            switch( recmode)
            {
            case MID811_REC_PES:
            case MID811_REC_MUX:
                CONNECT_NEW( CDShowHelper::FindPin(FALSE, SnapshotFilter), CDShowHelper::FirstDisconnectedPin(TRUE,Muxer), "snapshot to mux");
                break;
            default:
                break;
            }
            CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
        } else {
            // no snapshots
            switch( recmode)
            {
            case MID811_REC_PES:
            case MID811_REC_MUX:
                if( VideoTee == NULL) {
                    CONNECT_NEW( CDShowHelper::FindPin(RAW_CAPTURE_PINNAME, AVCapture), CDShowHelper::FirstDisconnectedPin(TRUE,Muxer), "Capture to mux");
                }
                else {
                    CONNECT_NEW( CDShowHelper::FirstDisconnectedPin(FALSE,VideoTee), CDShowHelper::FirstDisconnectedPin(TRUE, Muxer), "Capture to mux");
                }
                break;
            default:
                break;
            }
            CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
        }
    }

    if( m_bRecordAudio ) {
        assert(Muxer != NULL);
        if (recmode == MID811_REC_MUX) {
            CONNECT_NEW( CDShowHelper::FirstDisconnectedPin( FALSE, AudioTee), CDShowHelper::FirstDisconnectedPin(TRUE,Muxer), "aud to mux");
            CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
        }
    }

    assert( Muxer != NULL);
    CONNECT_NEW( CDShowHelper::FirstDisconnectedPin(FALSE,Muxer), CDShowHelper::FirstDisconnectedPin(TRUE,FileWriter), "mux to writer");
    CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);

    if( bAddAudioNull) {
        //OUTPUTDEBUGSTRING("adding audio NULL");
        ADDFILTER_NEW( "{C1F400A4-3F08-11D3-9F0B-006008039E37}", "Null Renderer", NullRenderer2);
        CHECK_ERROR(_T("CSensorayDevice:BSG\n"), hr);
        CONNECT_NEW( CDShowHelper::FindPin(AUDIO_PINNAME, AVCapture), CDShowHelper::FindPin(TRUE,NullRenderer2), "SRAY to Null");
        CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
   } 

    m_bDevicePresent = bDeviceFound;
    if( m_bRenderVideo)
        SetOwnerWindow( m_hwnd);
    QueryInterfaces();
    
    return hr;
}

BOOL IsWindows7()
{
    OSVERSIONINFO osvi;
    BOOL bIsWindows7orLater;
    ZeroMemory(&osvi, sizeof(OSVERSIONINFO));
    osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
    GetVersionEx(&osvi);
    bIsWindows7orLater = 
       ( (osvi.dwMajorVersion > 6) ||
       ( (osvi.dwMajorVersion == 6) && (osvi.dwMinorVersion >= 1) ));
    
    return bIsWindows7orLater;
}

// For 811NC product(no compression).
// graph flow 811 Capture-> renderer ( no snapshots)
//
// with snapshots
//                Capture->Tee->Renderer
//                            ->SG
//

HRESULT CSensorayDevice::Build811PreviewGraph()
{
    CAutoLock lock(&m_critSec);
    HRESULT hr = S_OK;
    BOOL bDeviceFound = FALSE;
    int iSH = GetSnapshotMethod();
    IPinPtr pPin1 = NULL;
    IPinPtr pPin2 = NULL;
    IBaseFilterPtr AVCapture;
    IBaseFilterPtr AudioTee;
    IBaseFilterPtr VideoTee;
    IBaseFilterPtr WaveOut;
    IBaseFilterPtr VideoRenderer;
    IBaseFilterPtr SnapshotFilter;
    IBaseFilterPtr NullRenderer;

    ::CoInitialize(NULL);
    ReleaseInterfaces();
    m_pGB = NULL;
    hr = m_pGB.CreateInstance( CLSID_FilterGraph);
    CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);

    hr = CDShowHelper::AddCategoryFilter(AM_KSCATEGORY_CAPTURE,AVSTREAMCAPTURENAME,m_pGB, &AVCapture, m_iInstance);
    CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
    bDeviceFound = TRUE;
    pPin2 = CDShowHelper::FindPin(TRUE, AVCapture);
    //If we find the analog video input pin in AVCapture filter
    if( pPin2 ) { 
		hr = AddCrossbar(AVCapture);
		CHECK_ERROR(TEXT("CSensorayDevice::"), hr);
    }
    SetInput(m_pDLL->m_param[m_iInstance].m_input);
    if( !m_bMute || m_bRecordAudio) { 
        SetCrossbarAudioMute(FALSE);
    }
    else {
        SetCrossbarAudioMute(TRUE);
    }

    // call update compression settings.
    // For NC 811, this just updates the preview pin to the correct resolution
    UpdateVideoSettings(TRUE, m_bSnapshotEnable);

    // VMR 9 default
	if (IsWindows7()) {
		ADDFILTER_NEW( "{51b4abf3-748f-4e3b-a276-c828330e926a}", "VMR 9", VideoRenderer);
		if(FAILED(hr)){
			//OUTPUTDEBUGSTRING("trying VMR7\n");
			hr = S_OK;
			ADDFILTER_NEW( "{6BC1CFFA-8FC1-4261-AC22-CFB4CC38DB50}", "Default Video Renderer", VideoRenderer);    
			if( FAILED(hr)) {
				AfxMessageBox(_T("DirectX 9 not installed or video card not capable"));
				return hr;
			}
			m_bVMR9 = FALSE;
			//OUTPUTDEBUGSTRING("added VMR7\n");
			ConfigVMR7( VideoRenderer);
			m_pVideoRenderer = VideoRenderer;
		}
		else {
			m_bVMR9 = TRUE;
			//OUTPUTDEBUGSTRING("added VMR9\n");
			hr= 0;
			assert( VideoRenderer != NULL);
			ConfigVMR9( VideoRenderer);
			m_pVideoRenderer = VideoRenderer;
		}
	} else {
		ADDFILTER_NEW( "{6BC1CFFA-8FC1-4261-AC22-CFB4CC38DB50}", "Default Video Renderer", VideoRenderer);    
		if(FAILED(hr)){
			//OUTPUTDEBUGSTRING("trying VMR7\n");
			hr = S_OK;
			ADDFILTER_NEW( "{51b4abf3-748f-4e3b-a276-c828330e926a}", "VMR 9", VideoRenderer);    
			if( FAILED(hr)) {
				AfxMessageBox(_T("DirectX 9 not installed or video card not capable"));
				return hr;
			}
			m_bVMR9 = TRUE;
			//OUTPUTDEBUGSTRING("added VMR7\n");
			ConfigVMR9( VideoRenderer);
			m_pVideoRenderer = VideoRenderer;
		}
		else {
			m_bVMR9 = FALSE;
			//OUTPUTDEBUGSTRING("added VMR9\n");
			hr= 0;
			assert( VideoRenderer != NULL);
			ConfigVMR7( VideoRenderer);
			m_pVideoRenderer = VideoRenderer;
		}
	}

    pPin1 = CDShowHelper::FindPin(TRUE, VideoRenderer);
    if (m_bSnapshotEnable) {
		IBaseFilterPtr AVIDecomp;
		//OUTPUTDEBUGSTRING("SRAY811: AVI decompressor\n");
		ADDFILTER_NEW( AVI_DECOMP_CLSID , "AVI Decompressor", AVIDecomp);
		CHECK_ERROR(_T("CSensorayDevice:BSG\n"), hr);
		ADDFILTER_NEW("{F8388A40-D5BB-11D0-BE5A-0080C706568E}","Infinite Pin Tee For Video", VideoTee);//Infinite Tee
        CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
        pPin2 = CDShowHelper::FindPin(RAW_CAPTURE_PINNAME, AVCapture);
        pPin1 = CDShowHelper::FindPin(TRUE,VideoTee);
        CONNECT_NEW( pPin2, pPin1, "cap to tee");
        CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
        CONNECT_NEW( CDShowHelper::FirstDisconnectedPin(FALSE, VideoTee), CDShowHelper::FindPin(TRUE, AVIDecomp),"tee to render");
        CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
        CONNECT_NEW( CDShowHelper::FirstDisconnectedPin(FALSE, AVIDecomp), CDShowHelper::FindPin(TRUE, VideoRenderer),"tee to render");
        CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);

	}
    else {
#if 1
        pPin2 = CDShowHelper::FindPin(RAW_CAPTURE_PINNAME, AVCapture);
        CONNECT_NEW( pPin2, pPin1, "cap to render");
#else
		IBaseFilterPtr AVIDecomp;
		//OUTPUTDEBUGSTRING("SRAY811: AVI decompressor\n");
		ADDFILTER_NEW( AVI_DECOMP_CLSID , "AVI Decompressor", AVIDecomp);
		CHECK_ERROR(_T("CSensorayDevice:BSG\n"), hr);
		pPin2 = CDShowHelper::FindPin(RAW_CAPTURE_PINNAME, AVCapture);
		CONNECT_NEW( pPin2, CDShowHelper::FindPin(TRUE, AVIDecomp), "preview to decomp");
        CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
		CONNECT_NEW( CDShowHelper::FindPin(FALSE, AVIDecomp), CDShowHelper::FindPin(TRUE, VideoRenderer), "decomp to render");
#endif	
        CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
    }
    // if recording audio OR rendering audio, use audio tee(connect the audio)
    if(!m_bMute) {
        ADDFILTER_NEW("{F8388A40-D5BB-11D0-BE5A-0080C706568E}","Infinite Pin Tee For Audio", AudioTee);//Infinite Tee
        CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
        CONNECT_NEW( CDShowHelper::FindPin(AUDIO_PINNAME, AVCapture),
                 CDShowHelper::FirstDisconnectedPin( TRUE, AudioTee),
                "Connecting Audio capture with Infinite Pin Tee"
                );
        CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
        hr = CDShowHelper::AddCategoryFilter(CLSID_AudioRendererCategory, "Default DirectSound Device", m_pGB, &WaveOut, 0);
        if(FAILED(hr)) {
            IBaseFilterPtr NullRenderer2;
            ADDFILTER_NEW( "{C1F400A4-3F08-11D3-9F0B-006008039E37}", "Null Renderer2", NullRenderer2);
            CONNECT_NEW( CDShowHelper::FirstDisconnectedPin(FALSE, AudioTee), CDShowHelper::FindPin(TRUE, NullRenderer2), "test");
            m_bAudioHWPresent = FALSE;
        } else {
            IBaseFilterPtr NullRenderer2;
            CONNECT_NEW( CDShowHelper::FirstDisconnectedPin(FALSE, AudioTee), CDShowHelper::FindPin(TRUE, WaveOut), "test");
            if(FAILED(hr)) {
                IBaseFilterPtr NullRenderer;
                ADDFILTER_NEW( "{C1F400A4-3F08-11D3-9F0B-006008039E37}", "Null Renderer2", NullRenderer2);
                CONNECT_NEW( CDShowHelper::FirstDisconnectedPin(FALSE, AudioTee), CDShowHelper::FindPin(TRUE, NullRenderer2), "test");
                m_bAudioHWPresent = FALSE;

            }
        }
    }
    if( m_bSnapshotEnable) {
        AM_MEDIA_TYPE mt;
        memset( &mt, 0, sizeof( AM_MEDIA_TYPE));
        mt.majortype = MEDIATYPE_Video;
        mt.formattype = GUID_NULL;
        m_bRAWSnapshots = TRUE;
        ADDFILTER_NEW( CLSID_DEFAULT_SG, "SRAY Video Decoder Snapshot", SnapshotFilter); 
        CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
        m_pISG = SnapshotFilter;
        // make sure buffering turned on 
        m_pISG->SetBufferSamples( TRUE);
        switch(m_colorspace) {
                case MID811_COLORSPACE_YUY2:
                default:
                    mt.subtype = MEDIASUBTYPE_YUY2;
                    break;
                case MID811_COLORSPACE_RGB32:
                    mt.subtype = MEDIASUBTYPE_RGB32;
                    break;                
                case MID811_COLORSPACE_RGB24:
                    mt.subtype = MEDIASUBTYPE_RGB24;
                    break;
                case MID811_COLORSPACE_UYVY:
                    mt.subtype = MEDIASUBTYPE_UYVY;
                    break;
        }
        m_pISG->SetMediaType( &mt);
        m_bRAWSnapshots = TRUE;

        CONNECT_NEW( CDShowHelper::FirstDisconnectedPin(FALSE, VideoTee),  CDShowHelper::FindPin(TRUE, SnapshotFilter), "Connecting capture to snapshot filter\n");
        CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
        ADDFILTER_NEW( "{C1F400A4-3F08-11D3-9F0B-006008039E37}", "Null Renderer", NullRenderer);
        CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
        CONNECT_NEW( CDShowHelper::FindPin(FALSE, SnapshotFilter), CDShowHelper::FindPin(TRUE, NullRenderer), "SRAY to Null");
        CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);
    }

    m_bDevicePresent = bDeviceFound;
    SetOwnerWindow( m_hwnd);
    QueryInterfaces();
    
    return hr;
}




// Builds a Directshow Filter Graph to capture UNcompressed streams using callback functions

HRESULT CSensorayDevice::BuildCaptureOnlySG_CB(int colorspace)
{
    CAutoLock lock(&m_critSec);
    HRESULT hr = S_OK;
    BOOL bCapAudio;
    int iAdvanced = GetAdvanced();
    // bAddAudioNull below
    IBaseFilterPtr AVCapture;
    IBaseFilterPtr AudioTee;
    IBaseFilterPtr VideoTee;
    IBaseFilterPtr NullRenderer;
    IBaseFilterPtr SGVid;  // Video/ PES sample grabber
    IBaseFilterPtr SGAudio;  // audio sample grabber( for split stream modes)
    IBaseFilterPtr NullRendererAudio;

    ::CoInitialize(NULL);
    ReleaseInterfaces();

    bCapAudio = FALSE;
    
    m_pGB = NULL;
    hr = m_pGB.CreateInstance( CLSID_FilterGraph);
    CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr);

    m_pfFileWriter = NULL;

    CDShowHelper::AddCategoryFilter(AM_KSCATEGORY_CAPTURE,AVSTREAMCAPTURENAME,m_pGB, &AVCapture, m_iInstance);
    m_p811Proxy = CDShowHelper::FindFilter(AVSTREAMCAPTURENAME, m_pGB); 


    //If we find the analog video input pin in AVCapture filter
    if( CDShowHelper::FindPin(TRUE, AVCapture) )
    {
        //Add Crossbar filter and connect it with AVCapture filter
		hr = AddCrossbar(AVCapture);
        CHECK_ERROR(_T("CSensorayDevice:BSG\n"), hr);
    }
    SetCrossbarAudioMute(FALSE);
    // call update compression settings.
    // For NC 811, this just updates the capture pin to the correct resolution
    UpdateVideoSettings(FALSE, TRUE);

    ADDFILTER_NEW("{F8388A40-D5BB-11D0-BE5A-0080C706568E}","Infinite Pin Tee For Video", VideoTee);//Infinite Tee
    CHECK_ERROR(_T("CSensorayDevice:BSG\n"), hr);
    CONNECT_NEW( CDShowHelper::FindPin( RAW_CAPTURE_PINNAME, AVCapture), CDShowHelper::FindPin( TRUE, VideoTee), " ");
    CHECK_ERROR(_T("CSensorayDevice:BSG\n"), hr);
    // add sample grabber 1
    ADDFILTER_NEW( CLSID_DEFAULT_SG, "Sample Grabber", SGVid); 
    CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr); 
    m_pISG_vid = SGVid;

    // set the colorspace
    {
        AM_MEDIA_TYPE mt;
        memset( &mt, 0, sizeof(AM_MEDIA_TYPE));
        mt.majortype = MEDIATYPE_Video;
        mt.formattype = GUID_NULL;
        switch (colorspace) {
                case MID811_COLORSPACE_UYVY:
                    mt.subtype = MEDIASUBTYPE_UYVY;
                    break;
                case MID811_COLORSPACE_RGB24:
                    mt.subtype = MEDIASUBTYPE_RGB24;
                    break;                
                default:
                case MID811_COLORSPACE_YUY2:
                    mt.subtype = MEDIASUBTYPE_YUY2;
                    break;
        }
        m_pISG_vid->SetMediaType( &mt);
    }
    // add null renderer
    ADDFILTER_NEW( "{C1F400A4-3F08-11D3-9F0B-006008039E37}", "Null Renderer", NullRenderer);
    CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr); 

    // if recmode 0, connect Uncompressed out to SG 
    CONNECT_NEW(CDShowHelper::FirstDisconnectedPin(FALSE, VideoTee), CDShowHelper::FindPin( TRUE, SGVid), "");
    CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr); 
    // sample grabber to null renderer
    CONNECT_NEW(CDShowHelper::FindPin(FALSE, SGVid), CDShowHelper::FindPin( TRUE, NullRenderer), "");
    CHECK_ERROR(TEXT("CSensorayDevice::BRPG"), hr); 
    m_pISG_vid->SetCallback( &m_vidCB,0);
    // query interfaces
    QueryInterfaces();
    SetInput(m_pDLL->m_param[m_iInstance].m_input);
    return hr;
}

HRESULT CSensorayDevice::AddCrossbar(IBaseFilterPtr AVCapture)
{
	HRESULT hr;
	IAMCrossbarPtr ptmp;
	ICaptureGraphBuilder2 *pGB2;
	hr = CoCreateInstance(CLSID_CaptureGraphBuilder2, NULL,
		CLSCTX_INPROC_SERVER, IID_ICaptureGraphBuilder2, 
		(void **)&pGB2);
	pGB2->SetFiltergraph(m_pGB);
	hr = pGB2->FindInterface(&PIN_CATEGORY_CAPTURE, &MEDIATYPE_Video, AVCapture, IID_IAMCrossbar, (void **)&ptmp);
	ptmp = NULL;
	if (pGB2) {
		pGB2->SetFiltergraph(NULL);
		pGB2->Release();
	}
	return hr;
}