#include "stdafx.h"
#include "dshowhelper.h"
#include "dmodshow.h"
#include "dmoreg.h"
#include "debug.h"
#include "assert.h"
#include "interfaces.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

HRESULT CDShowHelper::FindNumDevices( REFCLSID clsidCategory,CString strName, int *numDevices, IGraphBuilderPtr pGB)
{
	HRESULT hr = S_OK;
	ICreateDevEnumPtr pSysDevEnum = NULL;
	IEnumMonikerPtr pEnumCat = NULL;
	IMonikerPtr pMoniker = NULL;    
	IPropertyBagPtr pPropBag = NULL;
	ULONG cFetched;
	IBaseFilterPtr pFilter = NULL;
	CString	pinName;
    int num = 0;
    hr = CoCreateInstance(CLSID_SystemDeviceEnum, 
        NULL, CLSCTX_INPROC, IID_ICreateDevEnum, 
        (void **)&pSysDevEnum);
    
    CHECK_ERROR(TEXT("CDShowHelper %x"), hr);

    hr = pSysDevEnum->CreateClassEnumerator(clsidCategory, &pEnumCat, 0);
    CHECK_ERROR(TEXT("CDShowHelper %x"), hr);
    if(	pEnumCat == NULL)
    {
        hr = E_FAIL;

        pFilter = NULL;
        pPropBag = NULL;
        pMoniker = NULL;
        pEnumCat = NULL;
        pSysDevEnum = NULL;
        *numDevices = num;
        return hr;
    }
    while(pEnumCat->Next(1, &pMoniker, &cFetched) == S_OK)
    {
        pMoniker->BindToStorage(0, 0, IID_IPropertyBag, (void **)&pPropBag);
        VARIANT varName;
        varName.vt = VT_BSTR;
        hr = pPropBag->Read(L"FriendlyName", &varName, 0);
        if(SUCCEEDED(hr))
        {
            if(CString(varName.bstrVal) == strName)
            {
                num++;
            }
            SysFreeString(varName.bstrVal);
        }
        pPropBag = NULL;
        pMoniker = NULL;
    }
    pEnumCat = NULL;
    pSysDevEnum = NULL;

    pFilter = NULL;
    pPropBag = NULL;
    pMoniker = NULL;
    pEnumCat = NULL;
    pSysDevEnum = NULL;
    *numDevices = num;
    return hr;
}


HRESULT CDShowHelper::AddFilterDmo( CString strClsid, CString strName, IBaseFilter **ppFilter, IGraphBuilderPtr pGB)
{
// Create the DMO Wrapper filter.
    IBaseFilter*  pFilter;
    HRESULT hr = CoCreateInstance(CLSID_DMOWrapperFilter, NULL, 
            CLSCTX_INPROC_SERVER, IID_IBaseFilter, 
            reinterpret_cast<void**>(&pFilter));
	CLSID clsid;
	CLSIDFromString(strClsid.AllocSysString(),&clsid);

    if (SUCCEEDED(hr)) 
    {
        // Query for IDMOWrapperFilter.
        IDMOWrapperFilterPtr pDmoWrapper;
        hr = pFilter->QueryInterface(IID_IDMOWrapperFilter, 
            reinterpret_cast<void**>(&pDmoWrapper));

        if (SUCCEEDED(hr)) 
        {     
            // Initialize the filter.
            hr = pDmoWrapper->Init(clsid, DMOCATEGORY_VIDEO_EFFECT); 
            pDmoWrapper = NULL;
        }
        *ppFilter = pFilter;

    }
    return hr;
}


IBaseFilterPtr CDShowHelper::FindFilter(CString strName, IGraphBuilderPtr pGB)
{
    HRESULT hr = S_OK;
    IBaseFilterPtr pFilter = NULL;
    hr = pGB->FindFilterByName(strName.AllocSysString(), &pFilter);
    if(FAILED(hr))
    {
        CString strInfo;
        TCHAR szErr[MAX_ERROR_TEXT_LEN];
        DWORD res = AMGetErrorText(hr, szErr, MAX_ERROR_TEXT_LEN);
        if (res == 0)
        {
            wsprintf(szErr, _T("Unknown Error: 0x%2x"), hr);
        }
        strInfo.Format(_T("Failed Find Filter %s,\r\nError Code:0x%x,\r\n%s"),strName,hr,szErr);
        //OUTPUTDEBUGSTRING(strInfo);
        return NULL;
    }
    return pFilter;	
}

// same as SetSyncClock, but uses default routines instead of filter proxies
HRESULT CDShowHelper::SetSyncClockGB(IGraphBuilderPtr pGB, int RefClock)
{
	HRESULT hr = S_OK;
	
    IBaseFilterPtr pFilter = NULL;
	IReferenceClockPtr pClock = NULL;
	IMediaFilterPtr  pMedia =  NULL;
	// -1: Set clock to NULL. if no audio HW present
	if (RefClock == -1) {
		hr = pGB->QueryInterface(__uuidof(IMediaFilter),(void**)&pMedia);
		if(pMedia) {
            hr = pMedia->SetSyncSource(NULL);
			pMedia = NULL;
		}
		return hr;
	}
	// 0: default clock.
	if(RefClock == 0) {
		//OUTPUTDEBUGSTRING("default");
		hr = pGB->SetDefaultSyncSource();
		return hr;
	}

	//1: System Clock
	if(RefClock == 1) {
		hr = CoCreateInstance(CLSID_SystemClock, 
			NULL, CLSCTX_INPROC_SERVER, IID_IReferenceClock, 
			(void **)&pClock);
		if(FAILED(hr)) {
			OUTPUTDEBUGSTRING(_T("Failed to get system reference clock"));
            return hr;
		}
    } else if(RefClock == 2) {// audio source
		//OUTPUTDEBUGSTRING("audio");
		// 2 kinds of audio render is used, Default directsound device
		// or default waveout device.
		hr = pGB->FindFilterByName(L"Default DirectSound Device", &pFilter);
		if (NULL == pFilter) {
			pGB->FindFilterByName(L"Default WaveOut Device", &pFilter);
		}
		if(pFilter == NULL ) {
			//OUTPUTDEBUGSTRING("Cannot find Audio Renderer Device");
			return CLASS_E_CLASSNOTAVAILABLE;
		}
		hr = pFilter->QueryInterface(IID_IReferenceClock, (void **)&pClock);
		if(FAILED(hr)) {
			//OUTPUTDEBUGSTRING(_T("Failed Query Audio Reference Clock"));
			return hr;
		}		
    }
	if (pClock) {
		hr = pGB->QueryInterface(__uuidof(IMediaFilter),(void**)&pMedia);
		if(pMedia)
		{
			hr = pMedia->SetSyncSource(pClock);
			pMedia = NULL;
		}

		pClock = NULL;
	}
	return hr;
}

IPinPtr CDShowHelper::FindPin(CString strName, IBaseFilterPtr pFilter)
{
	HRESULT hr;
	ULONG cFetched = 1;
	IEnumPinsPtr pEnumPins;
	IPinPtr pPin =  NULL;
	
    hr = pFilter->EnumPins(&pEnumPins);

	if(FAILED(hr))
		return NULL;

	while (cFetched)
	{
		hr = pEnumPins->Next(1, &pPin, &cFetched);
		if(FAILED(hr) || pPin == NULL)
		{
			pEnumPins = NULL;
			return NULL;
		}
		PIN_INFO pi;
		hr = pPin->QueryPinInfo(&pi);
        if(FAILED(hr))
		{
			pEnumPins = NULL;
			pPin = NULL;
			return NULL;
		}
		pi.pFilter->Release();
		if ( strName.CompareNoCase(CString(pi.achName)) == 0)
		{
			pEnumPins = NULL;
			return pPin;
		}
		pPin = NULL;
	}
	pEnumPins = NULL;
	return NULL;
}


HRESULT CDShowHelper::AddCategoryFilter(REFCLSID clsidCategory,CString strName,IGraphBuilderPtr pGB, IBaseFilter** ppFilter, int instance)
{
	HRESULT hr = S_OK;
    int cur_instance = 0;
	ICreateDevEnumPtr pSysDevEnum = NULL;
	IEnumMonikerPtr pEnumCat = NULL;
	IMonikerPtr pMoniker = NULL;
	IPropertyBagPtr pPropBag = NULL;
	ULONG cFetched;
	IBaseFilter *pFilter = NULL; 
	CString	pinName;

	{
		hr = CoCreateInstance(CLSID_SystemDeviceEnum, 
			NULL, CLSCTX_INPROC, IID_ICreateDevEnum, 
			(void **)&pSysDevEnum);
		CHECK_ERROR(TEXT("CDShowHelper %x"), hr);

		hr = pSysDevEnum->CreateClassEnumerator(clsidCategory, &pEnumCat, 0);
		CHECK_ERROR(TEXT("CDShowHelper %x"), hr);
		if(	pEnumCat == NULL)
		{
            hr = E_FAIL;
            //pFilter = NULL;
            pPropBag = NULL;
            pMoniker = NULL;
            pEnumCat = NULL;
            pSysDevEnum = NULL;
        }

		while(pEnumCat->Next(1, &pMoniker, &cFetched) == S_OK)
		{
			pMoniker->BindToStorage(0, 0, IID_IPropertyBag, (void **)&pPropBag);
			VARIANT varName;
			varName.vt = VT_BSTR;
			hr = pPropBag->Read(L"FriendlyName", &varName, 0);
			if(SUCCEEDED(hr))
			{
				if(CString(varName.bstrVal) == strName)
				{
                    if( cur_instance == instance)
                    {
					    hr = pMoniker->BindToObject(0, 0, IID_IBaseFilter, (void **)&pFilter);
                        CHECK_ERROR(TEXT("CDShowHelper %x"), hr);
					    break;
                    }
                    cur_instance++;
				}
				SysFreeString(varName.bstrVal);
			}
			pPropBag = NULL;
			pMoniker = NULL;
		}
        pEnumCat = NULL;
        pSysDevEnum = NULL;

        if(pFilter == NULL)
        {
            hr = E_DEVICE_NOT_READY;
            //pFilter = NULL;
            pPropBag = NULL;
            pMoniker = NULL;
            pEnumCat = NULL;
            pSysDevEnum = NULL;
            return hr;
        }
        *ppFilter = pFilter;
    }
    hr = pGB->AddFilter(pFilter, strName.AllocSysString());
    CHECK_ERROR(TEXT("CDShowHelper %x"), hr);
    // if( pFilter) pFilter->Release(); // = NULL;
	pPropBag = NULL;
	pMoniker = NULL;
	pEnumCat = NULL;
	pSysDevEnum = NULL;
	return hr;
}


IPinPtr CDShowHelper::FindPin(BOOL bInput, IBaseFilterPtr pFilter)
{
	HRESULT hr;
	ULONG cFetched = 1;
	IEnumPinsPtr pEnumPins;
	IPinPtr pPin =  NULL;
	PIN_DIRECTION MyDir;
	PIN_DIRECTION dir = bInput?PINDIR_INPUT:PINDIR_OUTPUT;

	hr = pFilter->EnumPins(&pEnumPins);
	if(FAILED(hr))
		return NULL;
	while (cFetched)
	{
		hr = pEnumPins->Next(1, &pPin, &cFetched);
		if(FAILED(hr) || pPin == NULL)
		{
			pEnumPins = NULL;
			return NULL;
		}
		hr = pPin->QueryDirection(&MyDir);
		if(FAILED(hr))
		{
            pEnumPins = NULL;
            pPin = NULL;
			return NULL;
		}
		if ( dir == MyDir)
		{
            pEnumPins = NULL;
			return pPin;
		}
		pPin = NULL;
	}
    pEnumPins = NULL;
	
	return NULL;
}


IPinPtr CDShowHelper::FirstDisconnectedPin(BOOL bInput, IBaseFilterPtr pFilter)
{
	HRESULT hr;
	ULONG cFetched = 1;
	IEnumPinsPtr pEnumPins;
	IPinPtr pPin =  NULL;
	PIN_DIRECTION MyDir;
	PIN_DIRECTION dir = bInput?PINDIR_INPUT:PINDIR_OUTPUT;

	hr = pFilter->EnumPins(&pEnumPins);
	if(FAILED(hr))
    {
		return NULL;
    }
	while (cFetched)
	{
		hr = pEnumPins->Next(1, &pPin, &cFetched);
		if(FAILED(hr) || pPin == NULL)
		{
			pEnumPins = NULL;
			return NULL;
		}
		hr = pPin->QueryDirection(&MyDir);
		if(FAILED(hr))
		{
			pEnumPins = NULL;
			pPin = NULL;
			return NULL;
		}
		if ( dir == MyDir)
		{
			IPinPtr pConn = NULL;
			pPin->ConnectedTo(&pConn);
			if(pConn == NULL)
			{
				pEnumPins = NULL;
				return pPin;
			}
			else
			{
				pConn = NULL;
			}
		}
		pPin = NULL;
	}

	pEnumPins = NULL;
	return NULL;
}

