// image.cpp : Defines the entry point for the console application
// bmp save routines.  image merge routines. jpg save routines
// uses jpeg-6b driver.
// image.cpp
// routines for image conversion 
// Sensoray Company 2005-2009

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include "jpeg-6b\jpeglib.h"
#include "jpeg-6b\cdjpeg.h"     /* Common decls for cjpeg/djpeg applications */
#include "image.h"
#ifdef DEBUG_PROFILE
#include "pi_time.h"
#endif



/** Color space conversion function
 * Y  =  0.257R + 0.504G + 0.098B + 16
 * Cr =  0.439R - 0.368G - 0.071B + 128
 * Cb = -0.148R - 0.291G + 0.439B + 128
 *
 * or using integer math.
 *
 * Y  = ( 16842R + 33030G +  6422B + 1048576 ) / 65536
 * Cr = ( 28770R - 24117G -  4653B + 8388608 ) / 65536
 * Cb = ( -9699R - 19070G + 28770B + 8388608 ) / 65536
 *
 *  @param R red value
 *  @param G green value
 *  @param B blue value
 *  @return YCrCb triplet short
 */
unsigned short RGB2YCrCb16(int R, int G, int B )
{
    int Y  = ( 16842 * R + 33030 * G +  6422 * B + 1048576 ) >> ( 16 + 2 );
    int Cr = ( 28770 * R - 24117 * G -  4653 * B + 8388608 ) >> ( 16 + 3 );
    int Cb = ( -9699 * R - 19070 * G + 28770 * B + 8388608 ) >> ( 16 + 3 );
    
    Y  = Y  > ( 235 >> 2 ) ? ( 235 >> 2 ) : Y;
    Cr = Cr > ( 240 >> 3 ) ? ( 240 >> 3 ) : Cr;
    Cb = Cb > ( 240 >> 3 ) ? ( 240 >> 3 ) : Cb;
    
    Y  = Y  < ( 16 >> 2 ) ? ( 16 >> 2 ) : Y;    //lint !e701 shift left of signed 
    Cr = Cr < ( 16 >> 3 ) ? ( 16 >> 3 ) : Cr;   //lint !e701 shift left of signed 
    Cb = Cb < ( 16 >> 3 ) ? ( 16 >> 3 ) : Cb;   //lint !e701 shift left of signed 
    return ( Y << 10 ) + ( Cr << 5 ) + Cb;
}

/** converts RGB to YCrCb.  returns each YCrCb value separately
 *  @param R red value
 *  @param G green value
 *  @param B blue value
 *  @param pY pointer to converted Y value
 *  @param pCr pointer to converted Cr value
 *  @param pCb pointer to converted Cb value
 */
volatile void RGB2YCrCb(int R, int G, int B, unsigned char *pY, unsigned char *pCr, unsigned char *pCb )
{
    int Y  = ( 16842 * R + 33030 * G +  6422 * B + 1048576 ) >> ( 16 ); //lint !e702 shift right of signed ...
    int Cr = ( 28770 * R - 24117 * G -  4653 * B + 8388608 ) >> ( 16 ); //lint !e702 shift right of signed ...
    int Cb = ( -9699 * R - 19070 * G + 28770 * B + 8388608 ) >> ( 16 ); //lint !e702 shift right of signed ...
    
    Y  = Y  > ( 235 ) ? ( 235 ) : Y;
    Cr = Cr > ( 240 ) ? ( 240  ) : Cr;
    Cb = Cb > ( 240 ) ? ( 240  ) : Cb;

    Y  = Y  < ( 16 ) ? ( 16 ) : Y;      //lint !e734 loss of precision 31 bits to 8
    Cr = Cr < ( 128 ) ? ( 128 ) : Cr;   //lint !e734 loss of precision 31 bits to 8
    Cb = Cb < ( 128 ) ? ( 128 ) : Cb;   //lint !e734 loss of precision 31 bits to 8
    
    *pY = Y;
    *pCr = Cr;
    *pCb = Cb;
    return;
}


/** 65536 R = 76533(Y-16) + 104936 * (Cr-128)
 *65536 G = 76533(Y-16) - 53451(Cr-128) - 25703(Cb -128)
 * 65536 B = 76533(Y-16) + 132677(Cb-128)
* conversion YCrCb2RGB
* @param Y: y value
* @param Cr: Cr value
* @param Cb: Cb value
* @param *pR: returned R value
* @param *pG: returned G value
* @param *pB: returned B value
*/
void YCrCb2RGB(int Y, int Cr, int Cb, unsigned char *pR, unsigned char *pG, unsigned char *pB)
{
    int R,G,B;
   
    Y = Y - 16;
    Cr = Cr - 128;
    Cb = Cb - 128;
    
    R = (76533 * Y + 104936 * Cr) >> 16; 
    G = ((76533 * Y) - (53451 * Cr) - (25703 * Cb)) >> 16;   
    B = ((76533 * Y) + (132677 * Cb)) >> 16; 
    
    // Even with proper conversion, some values still need clipping.
    if (R > 255) R = 255;
    if (G > 255) G = 255;
    if (B > 255) B = 255;
    if (R < 0) R = 0;
    if (G < 0) G = 0;
    if (B < 0) B = 0;
    
    *pR = R;  //lint !e734 loss of precision
    *pG = G;  //lint !e734 loss of precision
    *pB = B;  //lint !e734 loss of precision
    return;
}


/** YCrCb2RGB_fp floating point YCrCb conversion
 * @param y y value
 * @param Cr Cr value
 * @param Cb Cb value
 * @param pR return Red value
 * @param pG pointer to G value
 * @param pB pointer to B value
 */
void YCrCb2RGB_fp(int y, int Cr, int Cb, unsigned char *pR, unsigned char *pG, unsigned char *pB)
{
    double r, g, b;
    y = y - 16;
    Cr = Cr - 128;
    Cb = Cb - 128;

    r = 1.164 * y + 1.596 * Cr;
    g = 1.164 * y - 0.813 * Cr - 0.391 * Cb;
    b = 1.164 * y + 2.018 * Cb;

    if( r > 255) r = 255;
    if( g > 255) g = 255;
    if( b > 255) b = 255;

    if( r < 0) r = 0;
    if( g < 0) g = 0;
    if( b < 0) b = 0;

    *pR = (unsigned char) r;
    *pG = (unsigned char) g;
    *pB = (unsigned char) b;
    return;
}


/** Invert field zero
 *  @param newf pointer to allocated space for new field( must be large enough
 *  @param size size of newf
 *  @param f0 pointer to field 0
 *  @param h0 height of field 0
 *  @param f1 pointer to field 1
 *  @param h1 height of field 1
 *  @param w  width of fields( full width in unsigned chars)
 *  @param pRetW pointer to image width (in Pixels).  Modify this to a new value if the returned image width is different. Initial value *pRetW = w/3
 *  @param pRetH pointer to image height. Modify this to a new value if the returned image height is different. Initial value *pRetH = h0 + h1
 *  @return 0 on success, -1 otherwise
 */
int inv_fieldzero( unsigned char *newf, int size,  unsigned char *f0, int h0, unsigned char *f1, int h1, int w, int *pRetW, int *pRetH)
{
    int i;
    int j;
    j = h0-1;
    for( i=0; i< h0; i++, j-=1)
    {
        memcpy( newf + j*w,     &f0[i*w], w);
    }
    return 0;
}

/** duplicate fields
 *  @param newf pointer to allocated space for new field( must be large enough
 *  @param size size of newf
 *  @param f0 pointer to field 0
 *  @param h0 height of field 0
 *  @param f1 pointer to field 1
 *  @param h1 height of field 1
 *  @param w  width of fields( full width in unsigned chars)
 *  @param pRetW pointer to image width (in Pixels).  Modify this to a new value if the returned image width is different. Initial value *pRetW = w/3
 *  @param pRetH pointer to image height. Modify this to a new value if the returned image height is different. Initial value *pRetH = h0 + h1
 *  @return 0 on success, -1 otherwise
 */
int dup_fields( unsigned char *newf, int size,  unsigned char *f0, int h0, unsigned char *f1, int h1, int w, int *pRetW, int *pRetH)
{
    int i;
    int j;
    j =0;
    for( i=0; i< h0; i++, j+=2)
    {
        memcpy( newf + j*w,     &f0[i*w], w);
        memcpy( newf + (j+1)*w, &f0[i*w], w);
    }
    return 0;
}




/** reverse the RGB ordering in a line.  Used for the JPEG library.  pline is changed by this function
 *  @param pline pointer to line
 *  @param length length of line
 *  @return none
 */
void reverse_rgb_line( unsigned char *pline, int length)
{
    int i;
    unsigned char tmp;
    
    for( i=0; i< length ;i+=3)
    {
        tmp = *(pline+i);
        *(pline+i) = *(pline+2+i);
        *(pline+2+i) = tmp;
    }
    return;
}


/** converts image buffer to JPG file specified
* @param image_buffer is a pointer to the image
* @param outfilename is the full path name of the resulting file
* @param info is the image info structure
* @param qual quality of jpeg.  25-100 where 100 is highest quality(largest file size)
* @return 0 on success, -ve on fail
*/
int image_to_jpg( const unsigned char *image, const wchar_t* outfilename, image_info_t *info, int qual )
{
    struct jpeg_compress_struct cinfo;
    struct jpeg_error_mgr jerr;
    FILE *fileout;
    int num_scanlines;
    int i;
    unsigned char *pline;
    wchar_t name[MAX_PATH];
#ifdef SENSORAY_OS_LINUX
    char aname[MAX_PATH];
#endif

    // open the file
    if( wcsstr(outfilename, L".jpg") == NULL)
    {
#ifndef SENSORAY_OS_LINUX
        _snwprintf(name, MAX_PATH, L"%s.%s", outfilename, L"jpg");
#else
        swprintf(name, MAX_PATH, L"%ls.%ls", outfilename, L"jpg");
#endif
    }
    else
    {
        wcsncpy(name, outfilename, MAX_PATH);
    }

    //OutputDebugString("image to jpg: ");
    //OutputDebugString(name);
    // open output file
#ifndef SENSORAY_OS_LINUX
    fileout = _wfopen( name, L"wb");
#else
    wcstombs(aname, name, MAX_PATH);
    fileout = fopen( aname, "wb");
#endif

    if( fileout == NULL)
    {
/*
#ifndef SENSORAY_OS_LINUX
        char szDebug[256];
        sprintf(szDebug, "error opening file: %lx\n", GetLastError());
        OutputDebugString(szDebug);
#endif
        */
        return -1;
    }
    
    /* Initialize the JPEG compression object with default error handling. */
    cinfo.err = jpeg_std_error(&jerr);
    jpeg_create_compress(&cinfo);

    // set RGB space
    cinfo.in_color_space = JCS_RGB; 
    
    // set the defaults
    jpeg_set_defaults(&cinfo);
    jpeg_default_colorspace(&cinfo);

    // set the destination out file
    jpeg_stdio_dest( &cinfo, fileout);  

    // initialize the jpeg params
    cinfo.image_height = info->height;
    cinfo.image_width = info->width;
    cinfo.input_components = 3;
    cinfo.in_color_space = (J_COLOR_SPACE) 2;
    cinfo.is_decompressor = 0;
    //cinfo.global_state = 100;
    cinfo.input_gamma = 1;
    cinfo.data_precision = 8;
    //cinfo.num_components = 3;
    //cinfo.jpeg_color_space =3;
    if( qual >100 )
        qual =100;
    else if( qual < 25)
        qual = 25;

    jpeg_set_quality( &cinfo, qual, TRUE);

    // start the Library compression
    jpeg_start_compress( &cinfo, TRUE);

    // start the compression
    num_scanlines = 1;  // one line at a time
    
    // write out all the lines
    pline = (unsigned char *) malloc( cinfo.image_width * 3 );
    if (pline == NULL)
        return -1;

#ifndef MID811_BUILD
    for (i=0; i < cinfo.image_height; i++)  // regular order
#else
    for (i= cinfo.image_height-1; i>=0; i--)  // reverse order
#endif
    {
        // reverse RGB --> BGR
        // not efficient, but safer to copy the line before reversing.
        memcpy( pline, &image[i*cinfo.image_width *3], cinfo.image_width * 3);
        reverse_rgb_line( pline, cinfo.image_width*3);
        (void) jpeg_write_scanlines(&cinfo, &pline, num_scanlines);
        // important, sleep while writing scanlines to free CPU for other tasks
        if( (i%20) == 0)
            Sleep(1);
    }
    free( pline);
    
    // finish up
    jpeg_finish_compress( &cinfo);
    jpeg_destroy_compress( &cinfo);

    // close the output file
    fclose( fileout);
    return 0;
}


/** saves an image to bmp file
 *  @param filename is the full file path to save( minus the extension)
 *  @param *image, pointer to image data in Y Cr Cb format
 *  @param type is the type( 0-- BMP,  1--ppm)
 *  @return 0 on success, -1 if file invalid
*/
int save_image_uncompressed(const unsigned char *image, const wchar_t *szFilename, image_info_t *info, int type)
{
    char ppmheader[256];
    wchar_t name[MAX_PATH];
#ifdef SENSORAY_OS_LINUX
    char    aname[MAX_PATH];
#endif
    FILE *fptr;
    int val;
    int rlen; // row length or number of columns( bytes== pixels * bytes per pixel)
    int clen; // column length or number of rows
    int bpp; // unsigned chars per pixel! not bits
    int i;
    
    rlen = info->stride;
    clen = info->height;
    bpp = info->stride / info->width;
    
    // open the file
#ifndef SENSORAY_OS_LINUX
    if (type == 0) 
        _snwprintf(name, MAX_PATH, L"%s.%s", szFilename, L"bmp");
    else
        _snwprintf(name, MAX_PATH, L"%s.%s", szFilename, L"ppm");
    fptr = _wfopen( name, L"wb");
#else
    if (type == 0) 
	    swprintf(name, MAX_PATH, L"%ls.%ls", szFilename, L"bmp");
    else
	    swprintf(name, MAX_PATH, L"%ls.%ls", szFilename, L"ppm");
    wcstombs(aname, name, MAX_PATH);
    fptr = fopen(aname, "wb");
#endif

    if (fptr == NULL)
        return -1;

    if (type == 1) {
        sprintf( ppmheader, "P6\n#ppm image\n%d %d\n255\n", info->width, info->height);
        fwrite( ppmheader, 1, strlen(ppmheader), fptr);
        // write out the rows
        for ( i=0; i< clen; i++)
        {
            fwrite( &image[rlen*i], 1, rlen, fptr);
        }
    } else {
        val = 0x4d42;                   fwrite(&val,1,2,fptr); //bmp signature
        val = (rlen * clen / bpp) + 54; fwrite(&val,1,4,fptr); //size of bmp file
        val = 0;                        fwrite(&val,1,2,fptr); //must be 0
        val = 0;                        fwrite(&val,1,2,fptr); //must be 0
        val = 54;                       fwrite(&val,1,4,fptr); //offset to image start
        val = 40;                       fwrite(&val,1,4,fptr); //must be 40
        val = rlen / bpp;               fwrite(&val,1,4,fptr); //image width in pixels
        val = clen ;                    fwrite(&val,1,4,fptr); //image height in pixels
        val = 1;                        fwrite(&val,1,2,fptr); //must be 1
        val = bpp * 8;                  fwrite(&val,1,2,fptr); //bits per pixel
        val = 0;                        fwrite(&val,1,4,fptr); //compression (none)
        val = (rlen * clen / bpp);      fwrite(&val,1,4,fptr); //size of image data
        val = 2835;                     fwrite(&val,1,4,fptr); //horizontal pixels/m
        val = 2835;                     fwrite(&val,1,4,fptr); //vertical pixels/m
        val = 0;                        fwrite(&val,1,4,fptr); //colors in image, or 0
        val = 0;                        fwrite(&val,1,4,fptr); //important colors,or 0
        
#ifndef MID811_BUILD
        // BMP reverse the image order. for 2246, retrieves in reverse order
        for (i= clen-1; i>= 0; i--)  //rows
        {
            fwrite( &image[i*rlen], 1, rlen, fptr);
        }
#else
        // MID811_BUILD, regular order
        for (i= 0; i<clen; i++)  //rows
        {
            fwrite( &image[i*rlen], 1, rlen, fptr);
        }

#endif

    }
 
    fclose( fptr);
    return 0;
}

/** converts yuy2 image to rgb image in memory
 *  @param in pointer to input image
 *  @param out pointer to output image
 *  @param in_end end of image
 *  @param stride stride of image
 *  @return 0 on success
 */
int yuy2_to_rgb_mem( const unsigned char *pin, unsigned char *pout, const unsigned char *pend, int stride)
{
    /* YUY2: Y0U0Y1V0Y2U1Y3V1 see DirectShow docs... */
	/* constants to do the conversion */
	ULONGLONG c10 =		0x00000000000100010i64;
	ULONGLONG c80 =		0x00080008000800080i64;
	ULONGLONG cff =		0x000FF00FF00FF00FFi64;
	ULONGLONG c20 =		0x00000200000002000i64;
	ULONGLONG cy =		0x000004A8500004A85i64;
	ULONGLONG crv =		0x03313000033130000i64;
	ULONGLONG cgu_cgv = 0x0E5FCF377E5FCF377i64;
	ULONGLONG cbu =		0x00000408D0000408Di64;

	int pass = 0;
	_asm{
		mov	eax, stride
		mov	esi,[pend]; read input from bottom
		mov	edi,[pout]; 
iter0:
		lea	ecx,[esi-8]
		; subtract stride(line width)
		mov eax, stride
		sub	esi,eax
iter1:
; 4 mmx registers per source dword
;image array of 2 little endian word values 
		movd	mm0,[esi]		; //mm0 holds Y0U0Y1V0
		movd	mm5,[esi+4]		; //mm5 holds	Y2U1Y3V1
		movq	mm1,mm0			
		pand	mm0,cff;		;// mm0 y0-y1  ( Y's and'd out
		movq	mm4,mm5			
		psrlw	mm1,8	;// u0 v0 
		pand	mm4,cff ;// y2 y3
		movq	mm2,mm5	; 
		psrlw	mm5,8	;// u1 v1
		psubw	mm0,c10;
		;// if second pass, subtract extra 4 to mm6 address below
		mov		eax,dword ptr[pass];
		shl		eax,2;
		mov		edx, 8
		sub		edx,eax
		movd	mm6,[esi+edx]    ;// line start + 8 
		psrlw	mm2,8	;// u2 v2
		psubw	mm4,c10 ; 
		paddw	mm2,mm1 ; 
		psrlw	mm6,8
		psubw	mm1,c80; 
		paddw	mm6,mm5
		psllq	mm2,32
		psubw	mm5,c80; 
		punpcklwd	mm0,mm2	;//mm0 Y1(upper DWORD) Y0( lower DWORD)
		psllq		mm6,32
		;// scale Y
		pmaddwd		mm0,cy; 
		punpcklwd	mm4,mm6
		paddw		mm1,mm1
		pmaddwd		mm4,cy
		paddw		mm5,mm5
		;// mm1 = 2 times UV values
		paddw		mm1,mm2	
		paddd		mm0,c20; 
		paddw		mm5,mm6
		movq		mm2,mm1
		paddd		mm4,c20;
		movq		mm3,mm1
		movq		mm6,mm5
		pmaddwd		mm1,crv
		movq		mm7,mm5
		paddd		mm1,mm0
		pmaddwd		mm5,crv
		;// red
		psrad		mm1,14		
		paddd		mm5,mm4
		pmaddwd		mm2,cgu_cgv
		psrad		mm5,14
		paddd		mm2,mm0
		pmaddwd		mm6,cgu_cgv
		;// green 
		psrad		mm2,14		
		paddd		mm6,mm4
		pmaddwd		mm3,cbu
		psrad		mm6,14
		paddd		mm3,mm0
		pmaddwd		mm7,cbu
		add			esi,8
		add			edi,12; 
		mov			edx,0;
		mov			eax,dword ptr[pass]
		cmp			eax,edx
		jne			skip_compare;
		cmp			esi,ecx
skip_compare:
		;// blue values
		psrad		mm3,14		
		paddd		mm7,mm4
		pxor		mm0,mm0
		psrad		mm7,14
		;// mm3 = green, blue
		packssdw	mm3,mm2		
		packssdw	mm7,mm6
		; red(lower dword)
		packssdw	mm1,mm0
		packssdw	mm5,mm0
		movq		mm2,mm3
		movq		mm6,mm7
		; //red blue
		punpcklwd	mm2,mm1		
		punpcklwd	mm6,mm5
		; //green unpacked
		punpckhwd	mm3,mm1
		punpckhwd	mm7,mm5
		movq		mm0,mm2
		movq		mm4,mm6
		;// red green blue values
		punpcklwd	mm0,mm3	
		punpcklwd	mm4,mm7
		psllq		mm0,16
		psllq		mm4,16
		; //second RGB values
		punpckhwd	mm2,mm3	
		punpckhwd	mm6,mm7
		; RGB complete
		packuswb	mm0,mm2	
		packuswb	mm4,mm6

		; move to 12 bytes
		psrlq		mm0,8		
		psllq		mm4,8		
		movd		[edi-12],mm0
		psrlq		mm0,32
		por			mm4,mm0
		movd		[edi-8],mm4
		psrlq		mm4,32
		movd		[edi-4],mm4
		jb			iter1
		// repeat loop, but set second pass flag
		mov			eax, dword ptr [pass]
		add			eax, 1
		mov			dword ptr[pass], eax 
		mov			edx, 1
		cmp			eax,edx
		je			iter1 ; // second pass
		
		; // second pass done, clear pass flag
		xor eax, eax;
		mov dword ptr[pass], eax; 
		; //subtract line stride from source index
		mov eax, stride
		sub	esi,eax
		; //compare to see if done 
		cmp	esi,dword ptr[pin]
		ja	iter0
		;// restore floating point
		emms
	} // asm
	return 0;
}


/** converts yuv image to rgb image in memory
 *  @param in pointer to input image
 *  @param out pointer to output image
 *  @param height height of image
 *  @param width width of image
 *  @return 0 on success
 */
int yuv2rgb_mem( const unsigned char *in, unsigned char *out, int height, int width)
{
    int i;
    int k;
    unsigned char y,Cr,Cb;
    unsigned char r,g,b;

    // convert the image
    for ( i = 0; i< height; i++) {
        
        for (k=0 ; k < (width *3); k+=3)
        {
            y = in[i*width*3 + k]; 
            Cb = in[i*width*3 + k + 1 ];
            Cr = in[i*width*3 + k + 2 ];
#ifndef USE_FP_CONVERSION
            YCrCb2RGB( y,Cr,Cb,&r,&g,&b);
#else
            YCrCb2RGB_fp(y, Cr, Cb, &r, &g, &b);
#endif
            out[i*width*3+k]  = r;
            out[i*width*3+k+1] = g;
            out[i*width*3+k+2] = b;
        }
    }
    return 0; 

}

