#ifndef IMAGE_H
#define IMAGE_H
// image.h : Defines the entry point for the console application.
// bmp save routines.  image merge routines. jpg save routines
// uses jpeg-6b driver.
#ifdef SENSORAY_OS_LINUX
#include "types.h"
#else 
#include <ctype.h>
#endif


#define MAX_IMAGE_HEIGHT (540 *2)   // HD
#define MAX_IMAGE_WIDTH  1920       // HD

#ifdef __cplusplus
extern "C" {
#endif
/** image information
 *  @param width width of image (bytes)
 *  @param height height of image (bytes)
 */
typedef struct 
{
    int stride;
    int width;
    int height;
} image_info_t;


/** converts image buffer to JPG file specified
* @param image_buffer is a pointer to the image
* @param outfilename is the full path name of the resulting file
* @param info is the image info structure
* @param qual quality of jpeg.  25-100 where 100 is highest quality(largest file size)
* @return 0 on success, -ve on fail
*/
int image_to_jpg( const unsigned char *imagebuffer, const wchar_t *outfilename, image_info_t *info, int qual );

/** saves an image to bmp file
 *  @param filename is the full file path to save( minus the extension)
 *  @param *image, pointer to image data in Y Cr Cb format
 *  @param type is the type( 0-- BMP,  1--ppm)
 *  @return 0 on success, -1 if file invalid
*/
int save_image_uncompressed(const unsigned char *image, const wchar_t *outfilename, image_info_t *info, int type);

/** converts yuv image to rgb image in memory
 *  @param in pointer to input image
 *  @param out pointer to output image
 *  @param height height of image
 *  @param width width of image
 *  @return 0 on success
 */
int yuv2rgb_mem( const unsigned char *in, unsigned char *out, int height, int width);
int yuy2_to_rgb_mem( const unsigned char *in, unsigned char *out, const unsigned char *end, int stride);

/** Color space conversion function
 * Y  =  0.257R + 0.504G + 0.098B + 16
 * Cr =  0.439R - 0.368G - 0.071B + 128
 * Cb = -0.148R - 0.291G + 0.439B + 128
 *
 * or using integer math.
 *
 * Y  = ( 16842R + 33030G +  6422B + 1048576 ) / 65536
 * Cr = ( 28770R - 24117G -  4653B + 8388608 ) / 65536
 * Cb = ( -9699R - 19070G + 28770B + 8388608 ) / 65536
 *
 *  @param R red value
 *  @param G green value
 *  @param B blue value
 *  @return YCrCb triplet short
 */
unsigned short RGB2YCrCb16(int R, int G, int B );
#ifdef __cplusplus
}
#endif
#endif
