// Copyright 2006 - 2009 Sensoray Company Inc
// mid811.cpp : 811 API functions.  Core functions called by API.

#include "stdafx.h"
#include "StdAfx.h"
#include "sensoraydevice.h"
#include <assert.h>
#include "videoacc.h"
#include "debug.h"

#include "mid811types.h"
#include "mid811func.h"
#include "mid811.h"
#include "debug.h"

#include "assert.h"
#include "registry.h"

extern char AVSTREAMCAPTURENAME[256];

// External functions.
extern "C"
{
    #include "image.h"
}

/*lint +e762*/ // redundant declare


#define RELEASEMEDIAEVENTPOINTER(mep)\
{\
    if(mep!=NULL)\
{\
    mep->SetNotifyWindow(NULL, 0, 0);\
    mep = NULL;\
}\
}

//
//  Note!
//
//      If this DLL is dynamically linked against the MFC
//      DLLs, any functions exported from this DLL which
//      call into MFC must have the AFX_MANAGE_STATE macro
//      added at the very beginning of the function.
//
//      For example:
//
//      extern "C" BOOL PASCAL EXPORT ExportedFunction()
//      {
//          AFX_MANAGE_STATE(AfxGetStaticModuleState());
//          // normal function body here
//      }
//
//      It is very important that this macro appear in each
//      function, prior to any calls into MFC.  This means that
//      it must appear as the first statement within the 
//      function, even before any object variable declarations
//      as their constructors may generate calls into the MFC
//      DLL.
//
//      Please see MFC Technical Notes 33 and 58 for additional
//      details.
//

/////////////////////////////////////////////////////////////////////////////
// CMid811Dll

#define S811_PREAMBLE(devid) \
{\
    if( devid < 0) \
        return MID811_ERR_NODEVICE; \
    if( devid >= MAX_DEVICES) \
        return MID811_ERR_NODEVICE; \
    if( theDLL.m_pDC[devid]) \
    { \
        if( !theDLL.m_pDC[devid]->GetDevicePresent()) \
            return MID811_ERR_NODEVICE; \
    } \
    else \
    { \
        return MID811_ERR_NODEVICE; \
    }\
    if(!theDLL.m_bOpen) \
    {\
        return MID811_ERR_NOTINITIALIZED;\
    }\
} 

// Windows
//
// The best place for CoInitializeEx is in DLL Main, but this is 
// not recommended per the link below
//
//http://msdn.microsoft.com/en-us/library/ms678543(VS.85).aspx
//Because there is no way to control the order in which in-process servers 
//are loaded or unloaded, do not call CoInitialize, CoInitializeEx,
//or CoUninitialize from the DllMain function.
//
//The end user application is responsible for CoUninitialize, not this DLL
// Calling CoUninitialize anywhere in this DLL is dangerous because we don't want
// to mess up any user threads which may call COM outside of our DLL


BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
                )
{
   switch (ul_reason_for_call)
   {
      case DLL_PROCESS_ATTACH:
		  break;
      case DLL_THREAD_ATTACH:
         break;
      case DLL_THREAD_DETACH:
		  break;
      case DLL_PROCESS_DETACH:
         break;
	  default:
		break;
    }
    return TRUE;
}
 
void CMid811Dll::CleanUp()
{
    int i;
    m_bInit = FALSE;


    for( i=0; i<m_iNumDevices; i++)
    {
        if( m_pDC[i])
        {
            if( m_pDC[i]->IsGraphRunning())
            {
                m_pDC[i]->StopGraph();
                
            }
            m_pDC[i]->m_pGB = NULL;
        }
    }
    try{
        for( i=0; i<m_iNumDevices; i++)
        {
            m_pDC[i]->CleanUp();
            SAFE_DELETE( m_pDC[i]);
        }
        
        RELEASEMEDIAEVENTPOINTER( m_pEventDevSurpriseRemoval);
    }
    catch( ...)
    {

    }
    m_bOpen = FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// CMid811Dll construction

CMid811Dll::CMid811Dll()
{
    // TODO: add construction code here,
    int i;
    // Place all significant initialization in InitInstance
    // internal use for debug
    for( i=0; i<MAX_DEVICES; i++)
    {
        m_pDC[i] = NULL;
        m_param[i].m_bLOWLatency = FALSE; 

        // default is line input
        //m_param[i].m_audioInput = MID811_AUDIO_LINE;
        // no mic boost
        //m_param[i].m_micBoost = 0;
        m_param[i].m_input = MID811_SOURCE_COMPOSITE_0;
        m_param[i].m_vidsys = MID811_VIDSYS_NTSC;
        m_param[i].m_AspectRatio = MID811_ASPECT_NONE;
        // set NTSC by default
        // set to composite by default

        m_param[i].m_input = MID811_SOURCE_COMPOSITE_0;
        m_param[i].m_hue = 0;
        m_param[i].m_bright = 50;
        m_param[i].m_contrast = 50;
        m_param[i].m_vsat = 50;
        m_param[i].m_recmode = MID811_REC_MUX;
        
    }
   m_bInit = FALSE;
    m_pEventDevSurpriseRemoval = NULL;
    m_bTest = FALSE;
    m_bOpen = FALSE;  
    m_iNumDevices = 0;
}

CMid811Dll::~CMid811Dll() 
{
#ifndef VB_BUILD 
    //OUTPUTDEBUGSTRING("destructor\n");
    
    try {
        int i;
        for( i=0; i< MAX_DEVICES; i++)
        {
            SAFE_DELETE( m_pDC[i]);
        }
        RELEASEMEDIAEVENTPOINTER( m_pEventDevSurpriseRemoval);
    }
    catch( ... )
    {
        OutputDebugString(_T("exception thrown CMid2250Destructor\n"));
    }
#endif
}



/*lint -e1762*/
int CMid811Dll::FindNumDevices()
{
    IGraphBuilderPtr pGB;
    int i;
    HRESULT hr;
    ::CoInitialize( NULL);
    pGB.CreateInstance(CLSID_FilterGraph);
    hr = CDShowHelper::FindNumDevices( AM_KSCATEGORY_CAPTURE, AVSTREAMCAPTURENAME, &i, pGB);
    if(!FAILED(hr)) {
        pGB = NULL;
        return i;
    }
    pGB = NULL;
    return 0;
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CMid811Dll object

CMid811Dll theDLL; // = NULL;


/////////////////////////////////////////////////////////////////////////////
// exported functions


/** initializes the middleware.  Give pointer to window in which to display video 
*   @param hwnd pointer to window in which to display video
*   @param bResize define to TRUE to resize the window
*    to the optimal native video size. Best performance with default TRUE
*   @return 0 on success
*/

MID811_API int __stdcall S811_Open(void)
{
    int numDevices = 0;
    int i;
    if( theDLL.m_bOpen)
    {
        OutputDebugString(_T("MID811 SDK/API already open\n"));
        return -1;
    }
    //allocate members
    for( i=0 ; i<MAX_DEVICES; i++)
    {
        assert( theDLL.m_pDC[i] == NULL); //assert not allocated yet
    }
    

    numDevices = theDLL.FindNumDevices();

    for( i=0; i<numDevices; i++)
    {
        theDLL.m_pDC[i] = new CSensorayDevice(&theDLL);
        theDLL.m_pDC[i]->m_iInstance = i;
        theDLL.m_pDC[i]->m_bSnapshotEnable = FALSE;
    }
    theDLL.m_bInit = TRUE;
    theDLL.m_bOpen = TRUE;
    theDLL.m_iNumDevices = numDevices;
    return 0;
}

MID811_API int __stdcall S811_SetRemoveMsg( HWND messageHwnd, int removeMsg, int devid)  
{
    HRESULT hr = S_OK;
    S811_PREAMBLE(devid);

    if( messageHwnd != NULL)
    {
        theDLL.m_pEventDevSurpriseRemoval = theDLL.m_pDC[devid]->m_pGB;
        if(theDLL.m_pEventDevSurpriseRemoval)
        {
            hr =theDLL.m_pEventDevSurpriseRemoval->SetNotifyWindow((OAHWND)messageHwnd, removeMsg, 0);
        }
        else return E_FAIL;
    }
    return hr;
}


/** S811_Close cleans up and shuts down the API gracefully
*/
MID811_API int __stdcall S811_Close(void)  
{
    /*lint -e506*/
    S811_PREAMBLE(0); //lint !e774
    theDLL.CleanUp();
    return 0;
}


/** set the video position in the clipping window 
*   @param xpos x position of video
*   @param ypos y position of video
*   @param xsize size of video display
*   @param ysize size of video display
*   @param left_clip number of pixels at left of source video to clip
*   @param top_clip y number of pixels at top of source video to clip
*   @param right_clip number of pixels at right of source video to clip
*   @param bottom_clip number of pixels at bottom of source video to clip
*   @return 0 on success
*/
MID811_API int __stdcall S811_SetVideoPosition(int xpos, int ypos, int xsize, int ysize, int left_clip, int top_clip, int right_clip, int bottom_clip, int devid)
{
    S811_PREAMBLE(devid);
    HRESULT hr = S_OK;
    RECT rcDst;
    RECT rcClip;

    rcDst.left   = xpos;
    rcDst.top    = ypos;
    rcDst.right  = xpos + xsize;
    rcDst.bottom = ypos + ysize;

    rcClip.left   = left_clip;
    rcClip.top    = top_clip;
    rcClip.right  = right_clip;
    rcClip.bottom = bottom_clip;

    theDLL.m_pDC[devid]->SetVideoPosition( rcDst, rcClip);
    return hr;
}


/** set the window handle where video is output from devid
*/
MID811_API int __stdcall S811_SetPreviewWindow( HWND hwnd, int devid)
{
    S811_PREAMBLE(devid);
    // because we are setting a display window, turn on rendering if not already
    if( !theDLL.m_pDC[devid]->GetRenderVideo()) {
        theDLL.m_pDC[devid]->SetRenderVideo( TRUE);
    }
    theDLL.m_pDC[devid]->SetOwnerWindow( hwnd);
    return 0;
}

/**
* Starts recording  to file using filter graph previously set up.
* @param filename is the filename to record to.  Use the full path, no  extension
* @param size is the size of the files  to record in Megabytes
*         if size full, then a new file started with an incremental number appended 
*         to the filename
* @param breakonrecsize flag indicating whether to, 1=end recording when filesize reaches size, or  0=continue recording to new file.
* @return 0 on  success
*/
MID811_API int __stdcall S811_StartRecordW(const wchar_t *filename, int devid)
{
    S811_PREAMBLE(devid);
    HRESULT hr = 0;
    if (theDLL.m_pDC[devid]->IsGraphRunning()) {
        // stop graph 
        theDLL.m_pDC[devid]->StopGraph();
    }
    hr = theDLL.m_pDC[devid]->BuildRecordGraph(filename);
    if( FAILED(hr))
        return hr; 
    hr = theDLL.m_pDC[devid]->StartGraph(); 
    theDLL.m_pDC[devid]->m_bIsRecording = TRUE;
    return hr;
}

MID811_API int __stdcall S811_StartRecord(const char *filename, int devid)
{
    int hr;
    LPWSTR pFilename;
	USES_CONVERSION;
	pFilename = A2OLE(filename);
    hr = S811_StartRecordW(pFilename, devid);
    return hr;
}


/** stop recording 
*/
MID811_API int __stdcall S811_StopRecord(int devid)
{
    S811_PREAMBLE(devid);
    HRESULT hr = S_OK;
    theDLL.m_pDC[devid]->StopGraph();
    theDLL.m_pDC[devid]->CleanUp();
    theDLL.m_pDC[devid]->m_bIsRecording = FALSE;
    return hr;
} 

/** pause recording
*/
MID811_API int __stdcall S811_PauseRecord(int devid)
{
    S811_PREAMBLE(devid);
    HRESULT hr;
    hr = theDLL.m_pDC[devid]->PauseGraph();
    return hr;
}

/** resume recording 
*/
MID811_API int __stdcall S811_ResumeRecord(int devid)
{
    S811_PREAMBLE(devid);
    HRESULT hr;
    hr = theDLL.m_pDC[devid]->ResumeGraph();
    return hr;
}



/** starts the streaming of video and audio 
*/
MID811_API int __stdcall S811_StartPreview(int devid) 
{ 
    HRESULT hr = S_OK;
    S811_PREAMBLE(devid);
    /* if display already running, return error */
    if( theDLL.m_pDC[devid]->IsGraphRunning())
    {
        return MID811_ERR_RUNNING;
    }
    hr = theDLL.m_pDC[devid]->BuildPreviewGraph();
    if(FAILED(hr))
    {
        //PopUpMessage("Filter graph cannot be built correctly. \nPlease check the validity of the configurations.");
        return hr;
    }

    // start the preview Graph
    hr = theDLL.m_pDC[devid]->StartGraph();
    return hr; 
}

/** stops streaming 
*/
MID811_API int __stdcall S811_StopPreview(int devid)
{
    S811_PREAMBLE(devid);
    if( !theDLL.m_pDC[devid]->IsGraphRunning())
        return -1;
    // next, stop streaming 
    theDLL.m_pDC[devid]->StopGraph();
    // cleanup
    theDLL.m_pDC[devid]->CleanUp();
    return 0;
}


/** Get video status of current channel
*   @return TRUE if locked otherwise 0
*/
MID811_API int __stdcall S811_GetVidStatus(int devid)
{
    S811_PREAMBLE(devid);  
    //    return theDLL.m_pDC[devid]->CheckHorizontalLock();
    return -1; // TBD
}


/** set the current video system
*   @param vidsys is the video system type
*   @return 0 on success
*/
MID811_API int __stdcall S811_SetVidSys(MID811_VIDSYS vidsys, int devid)
{
    HRESULT hr;
    S811_PREAMBLE(devid);
    theDLL.m_param[devid].m_vidsys = vidsys;
    // fix for CR( vidsys, source order problem)
    theDLL.m_pDC[devid]->SetInput( theDLL.m_param[devid].m_input);
    hr = theDLL.m_pDC[devid]->UpdateVideoSettings(FALSE,FALSE); 
    if( FAILED(hr)) {
        return hr;
    }
    return 0;
}

/** get the current video system
*   @param vidsys is the video system type
*   @return 0 on success
*/
MID811_API int __stdcall S811_GetVidSys(MID811_VIDSYS *vidsys, int devid)
{
    S811_PREAMBLE(devid);
    *vidsys = theDLL.m_param[devid].m_vidsys;
    return 0;
}

/** Sets the video input source. Note: will stop the stream when input changed.
*   @param input is the enumerated input MID811_SOURCE enum(composite, svideo, DV) (see mid811const.h)
*   @param bTest whether to include colorbars or not
*   @return 0 on success
*/
int CMid811Dll::SetSource(MID811_SOURCE input, int devid)
{
    // stop the display first( unsure if this is necessary.  leave for now)

    // call driver callback to set new source
    m_pDC[devid]->SetInput( input);

    // set the input in the API
    m_param[devid].m_input = input;

    return 0;
}

/** Sets the video input source. Note: will stop the stream when input changed.
*   @param input is the enumerated input MID811_SOURCE enum(composite, svideo, DV) (see mid811const.h)
*   @param bTest whether to include colorbars or not
*   @return 0 on success
*/
MID811_API int __stdcall S811_SetSource(MID811_SOURCE input, int devid)
{
    S811_PREAMBLE(devid); // don't return when m_bInit=0 because might be DV

    return theDLL.SetSource( input, devid);
}


/** Gets the current video source
*   @param pSource pointer to returned value of current input source
*   @return 0 on success 
*/
MID811_API int __stdcall S811_GetSource(MID811_SOURCE *pSource, int devid )
{
    S811_PREAMBLE(devid);
    *pSource = theDLL.m_param[devid].m_input;
    return 0;
}

/** get statistics ascii
*/
MID811_API int __stdcall S811_GetStatus(MID811STATUS *pStats, int devid )
{
    WIN32_FILE_ATTRIBUTE_DATA fileAttr;
    ULONG ulFileSize;
    LPSTR strPath;
    S811_PREAMBLE(devid);
    USES_CONVERSION;
    strPath = OLE2A(theDLL.m_pDC[devid]->m_curFilePath);
    if (strPath == NULL)
        return -1;
    strncpy(pStats->szFilepath, strPath, MAX_PATH);

    pStats->bIsPlaying = theDLL.m_pDC[devid]->IsGraphRunning();

    pStats->bIsRecording = theDLL.m_pDC[devid]->m_bIsRecording;
    if( GetFileAttributesExW( theDLL.m_pDC[devid]->m_curFilePath, GetFileExInfoStandard, &fileAttr))
    {
        // get size in megs
        ulFileSize = ( (fileAttr.nFileSizeLow / 1000) / 1000) + (fileAttr.nFileSizeHigh * ((0xffffffffUL / 1000) / 1000));
    }
    else
    {
        ulFileSize = 0;
    }
    pStats->iFilesize = ulFileSize;
    pStats->bRecordAudio = theDLL.m_pDC[devid]->GetRecordAudio();
    pStats->bRecordVideo = theDLL.m_pDC[devid]->GetRecordVideo();

    return 0;
}

/** get statistics unicode
*/
MID811_API int __stdcall S811_GetStatusW(MID811STATUS_W *pStats, int devid )
{
    WIN32_FILE_ATTRIBUTE_DATA fileAttr;
    ULONG ulFileSize;
    S811_PREAMBLE(devid);

    //strncpy( pStats->szFilepath, theDLL.m_pCM[devid]->m_curFilePath, MAX_PATH);
    wcsncpy( pStats->szFilepath, theDLL.m_pDC[devid]->m_curFilePath, MAX_PATH);
    pStats->bIsPlaying = theDLL.m_pDC[devid]->IsGraphRunning();

    pStats->bIsRecording = theDLL.m_pDC[devid]->m_bIsRecording;
    if( GetFileAttributesExW( theDLL.m_pDC[devid]->m_curFilePath, GetFileExInfoStandard, &fileAttr))
    {
        // get size in megs
        ulFileSize = ( (fileAttr.nFileSizeLow / 1000) / 1000) + (fileAttr.nFileSizeHigh * ((0xffffffffUL / 1000) / 1000));
    }
    else
    {
        ulFileSize = 0;
    }
    pStats->iFilesize = ulFileSize;
    pStats->bRecordAudio = theDLL.m_pDC[devid]->GetRecordAudio();
    pStats->bRecordVideo = theDLL.m_pDC[devid]->GetRecordVideo();

    return 0;
}


MID811_API int __stdcall S811_GetNumDevices(int *pBoards)
{
    /*lint -e506*/
    S811_PREAMBLE(0);  //lint !e774
    *pBoards = theDLL.m_iNumDevices;
    return 0;
}


/** 
*   Mutes the audio on the host computer.  Audio will still be recorded.  
*   Note: if currently streaming(StartStream used), this function will stop the stream and restart it.
*
*   @param bMute whether to mute audio on PC
*   @return 0 on success
*/
MID811_API int __stdcall S811_SetMute( BOOL bMute, int devid)
{
    S811_PREAMBLE(devid);
    theDLL.m_pDC[devid]->SetMute(bMute);
    return 0;
}
/** 
* Gets  the mute setting
*
* @param bMute  pointer to retrieved mute setting
* @return 0 on  success 
*/
MID811_API int __stdcall S811_GetMute( BOOL *bMute, int devid )
{
    BOOL mute;
    S811_PREAMBLE(devid);
    mute = theDLL.m_pDC[devid]->GetMute();
    *bMute = mute;
    return 0;
}

// turn on/off video recording
// NOTE: this is different from startrecord, etc.
// The 2 functions below are to allow configuration for either video or Audio 
// Elementary stream while recording.
// If RecordVideoStream is turned off, no video stream is sent to the recording
// module.
// If RecordAudioStream is turned off, no audio is sent to the recording module.(different from mute)
MID811_API int __stdcall S811_SetRecordVideoStream ( BOOL bRecVid, int devid)
{   
    S811_PREAMBLE(devid);

    theDLL.m_pDC[devid]->SetRecordVideo( bRecVid);
    return 0;
}

MID811_API int __stdcall S811_GetRecordVideoStream ( BOOL *recvid, int devid )
{
    S811_PREAMBLE(devid);
    *recvid = theDLL.m_pDC[devid]->GetRecordVideo();
    return 0; 
}

MID811_API int __stdcall S811_SetRecordAudioStream ( BOOL bRecAud, int devid)
{
    S811_PREAMBLE(devid);
    theDLL.m_pDC[devid]->SetRecordAudio( bRecAud);
    return 0;
}

MID811_API int __stdcall S811_GetRecordAudioStream ( BOOL *bRecAud, int devid)
{
    S811_PREAMBLE(devid);
    *bRecAud = theDLL.m_pDC[devid]->GetRecordAudio();
    return 0;
}


/** SetRenderVideo allows one to turn off the video window but still stream
*   the video after S811_StartStream 
*/
MID811_API int __stdcall S811_SetRenderVideo( BOOL bVideo, int devid)
{
    S811_PREAMBLE(devid);
    theDLL.m_pDC[devid]->SetRenderVideo( bVideo);
    return 0;
}

MID811_API int __stdcall S811_GetRenderVideo( BOOL *bVideo, int devid )
{
    S811_PREAMBLE(devid);
    *bVideo = theDLL.m_pDC[devid]->GetRenderVideo();
    return 0;
}

/** Repaint callback should be called when application receives a WM_PAINT event.  This is necessary to
*   notify the video renderer of a repaint event. 
*   @param hdc device context handle
*   @return 0 on success
*/
MID811_API int __stdcall S811_Repaint( HDC hdc)
{
    HRESULT hr = S_OK;
    int i;
//   AFX_MANAGE_STATE(AfxGetStaticModuleState());
    if(!theDLL.m_bOpen) 
    {
        return MID811_ERR_NOTINITIALIZED;
    }
    
    if(hdc != NULL) {
        for( i=0; i<theDLL.m_iNumDevices; i++) {
            if( theDLL.m_pDC[i]){
                int hr_local;
                hr_local = theDLL.m_pDC[i]->RepaintVideo( hdc);
                if( FAILED(hr_local)) {
                    hr = hr_local;
                }
            }
        }
   }
    else
    {
        HDC hdc2;
        for( i=0; i<theDLL.m_iNumDevices; i++) {
            hdc2 = GetDC( theDLL.m_pDC[i]->m_hwnd );
            if( theDLL.m_pDC[i]) {
                int hr_loc;
                hr_loc = theDLL.m_pDC[i]->RepaintVideo( hdc2);
                if( FAILED(hr_loc)) {
                    hr = hr_loc;
                }
            }
        }
    }
    return hr;
}

/** Informs the video renderer that a WM_DISPLAYCHANGE message has been received by the application
*/
MID811_API int __stdcall S811_DisplayChange()
{
    HRESULT hr = S_OK;
    HRESULT hr_loc;
    int i;

//   AFX_MANAGE_STATE(AfxGetStaticModuleState());

    if(!theDLL.m_bOpen) 
    {
        return MID811_ERR_NOTINITIALIZED;
    }

    for( i=0; i<theDLL.m_iNumDevices; i++)
    {
        hr_loc = theDLL.m_pDC[i]->DisplayModeChanged();
        if( FAILED(hr_loc))
        {
            hr = hr_loc;
        }
    }
    return hr;
}

/** Get the aspect ratio mode
*   @param mode returned aspect ratio mode
*   @return 0 on success
*/

MID811_API int __stdcall S811_GetAspectRatio(MID811_ASPECT_MODE *aspect, int devid)
{
    S811_PREAMBLE(devid);
    *aspect = theDLL.m_param[devid].m_AspectRatio;
    return 0; 
}

/** set aspect ratio flag 
*   @param mode MID811_ASPECT_NONE--stretched,  MID811_ASPECT_CONST--maintain aspect ratio
*   @return 0 on success, -ve on error
*/
MID811_API int __stdcall S811_SetAspectRatio(MID811_ASPECT_MODE mode, int devid)
{
    S811_PREAMBLE(devid);
    HRESULT hr = 0;
    if (theDLL.m_pDC[devid]) {
        hr = theDLL.m_pDC[devid]->SetAspectRatioMode( mode);
    }

    theDLL.m_param[devid].m_AspectRatio = mode;
    return hr;
}
/** set the brightness, contrast and saturation
*   @param param is the parameter to change(MID811_LEVEL_CONTRAST, MID811_LEVEL_BRIGHTNESS, MID811_LEVEL_SATURATION, MID811_LEVEL_HUE)
*   @param value is the value of the parameter
*   @return 0 on success
*/
MID811_API int __stdcall S811_SetLevels(int param, int value, int devid)
{
    HRESULT hr = S_OK;
    S811_PREAMBLE(devid); 
    if( param & MID811_LEVEL_CONTRAST )
    {
        hr = theDLL.m_pDC[devid]->SetContrast( value );
        if( FAILED(hr))
            return hr;
        theDLL.m_param[devid].m_contrast = value;
    }
    if( param & MID811_LEVEL_BRIGHTNESS )
    {
        hr = theDLL.m_pDC[devid]->SetBrightness( value );
        if( FAILED(hr))
            return hr;
        theDLL.m_param[devid].m_bright = value;
    }
    if( param & MID811_LEVEL_SATURATION )
    {
        hr = theDLL.m_pDC[devid]->SetSaturation( value);
        if( FAILED(hr))
            return hr;
        theDLL.m_param[devid].m_vsat = value;
    }
    if( param & MID811_LEVEL_HUE )
    {
        hr = theDLL.m_pDC[devid]->SetHue( value );
        if( FAILED(hr))
            return hr;
        theDLL.m_param[devid].m_hue = value;
    }
    return hr;
}

/** set the brightness, contrast and saturation
*   @param param is the parameter to change(MID811_LEVEL_CONTRAST, MID811_LEVEL_BRIGHTNESS, MID811_LEVEL_SATURATION, MID811_LEVEL_HUE)
*   @param value is the value of the parameter
*    ( CONTRAST, BRIGHTNESS, SATURATION: 0-100, default 50)
*    ( HUE: -50 to 50, default 0)
*   @return 0 on success
*/
MID811_API int __stdcall S811_GetLevel(int param, int *value, int devid)
{
    S811_PREAMBLE(devid);
    if( param & MID811_LEVEL_CONTRAST )
    {
        *value = theDLL.m_param[devid].m_contrast;
        return 0;
    }
    if( param & MID811_LEVEL_BRIGHTNESS )
    {
        *value = theDLL.m_param[devid].m_bright;
        return 0;
    }
    if( param & MID811_LEVEL_SATURATION )
    {
        *value = theDLL.m_param[devid].m_vsat;
        return 0;
    }
    if( param & MID811_LEVEL_HUE )
    {
        *value = theDLL.m_param[devid].m_hue;
        return 0;
    }
    return 0;
}



// returns frame in pFrame
// IN lSize -- size of pFrame
// OUT pBMI stores some(not all currently) of the bitmap info
//    pBMI->biWidth
//    pBMI->biHeight
//    pBMI->biBitCount
//    pBMI->biSizeImage
//    pBMI->biBitCount ( bits per pixel.  Stride == Width * biBitCount / 8)
// IN devid (devid number)
MID811_API int __stdcall S811_GetFrame( long inSize, unsigned char *pFrame, BITMAPINFOHEADER *pBMI, int devid)
{
    S811_PREAMBLE( devid);
    BITMAPINFOHEADER bmi;
    HRESULT hr;
    if(!theDLL.m_pDC[devid]->IsGraphRunning()) {
        return -1;
    }
    if( !theDLL.m_pDC[devid]->m_bSnapshotEnable) {
        return -1;
    }
    hr =  theDLL.m_pDC[devid]->GetSnapshotBufferStatic( &inSize, pFrame,  0, &bmi);
    if( FAILED( hr)) { 
        *pBMI = bmi;
        return hr; 
    }
    *pBMI = bmi;
    if( inSize != 0) {
        //OUTPUTDEBUGSTRING2(_T("got image %x %d\n"), hr, *inSize);
    }
    return 0;
}


/** takes a snapshot and saves to file in filename.  If MOTION_JPEG(MJPEG), this function will block until frame ready
*   @param szFilename is the fully qualified file + path( without extension)
*   @param filetype is the file type to save( see mid811const.h: MID811_FILE_JPEG and/or MID811_FILE_BMP)
*   @param freezetime is the time, in milli-sec to freeze the image. (minimum value greater than the video frame rate is required)
*   @param wait  wait =1 will wait if operations pending(devid locked), 
*                    wait=0 returns error if devid busy
*   @param qual-- jpeg quality, (use 25-100, default 100).
*   @return number of bytes read (image size) or -1 on error.
*/ 
MID811_API int __stdcall S811_SnapshotToFileW( const wchar_t *szFilename, int filetype, int qual, int devid)
{
    S811_PREAMBLE(devid);
    HRESULT hr;
    BYTE *pImage = NULL;
    BYTE *pWorkImage = NULL; 
    //long *plBuffer = NULL;
    long *pBuffer = NULL;
    long lSize;
    image_info_t info;
    int stride;
    unsigned char *pEnd;
    BITMAPINFOHEADER bmi;

#ifdef DEBUG_DUMP_RAW
    FILE *file;
#endif
    if(!theDLL.m_pDC[devid]->IsGraphRunning()) {
        return MID811_ERR_RUNNING;
    }
    if( !theDLL.m_pDC[devid]->m_bSnapshotEnable) {
        return -1;
    }
    hr = theDLL.m_pDC[devid]->GetSnapshotBuffer( &lSize, (void **)&pBuffer, &bmi);
    if( FAILED( hr )) {
        return hr;
    }
 
    if( pBuffer == NULL) {
        return -1;
    }

#ifdef DEBUG_DUMP_RAW
    // we have the image data, convert it to rgb
    // image_yuv is the same as image_rgb for efficiency here in pointer == out pointer
    FILE *file;
    file = fopen( "c:\\snapshot.bin", "wb+");
    if( file == NULL)
        return -1;
    fwrite( (void *) pBuffer, 1, lSize, file);
    fclose(file);
    file = fopen( "c:\\bitmapinfo.bin", "wb+");
    if( file == NULL)
        return -1;
    fwrite( (void *) &bmi, 1, sizeof(bmi) , file);
    fclose(file);

#endif

    stride = bmi.biWidth * bmi.biBitCount / 8;
    info.height = bmi.biHeight;
    info.width = bmi.biWidth;

    // allocate destination image if need to convert YUV style to RGB
    // currently support YUY2, otherwise assume RGB and save without
    // conversion.(This way if other conversions are necessary, they 
    // can be added later when the
    // the above is the case with the 811
    if( bmi.biCompression == mmioFOURCC('Y','U','Y','2'))
    {
        pWorkImage = (BYTE *) malloc(lSize*4);
        // point to end of image
        pEnd = ((BYTE *) pBuffer ) + stride * info.height;
        info.height = info.height;
        yuy2_to_rgb_mem((const unsigned char *) pBuffer, pWorkImage, pEnd, stride);
        pImage = pWorkImage;
    }
    else
    {
        // RGB8 (check explicitly later)
        pImage = (BYTE *)pBuffer;
#ifdef SAVE_RAW_DATA
        FILE *file;
        file = fopen("c:\\testimage.yuv", "wb");
        OutputDebugString("saving raw data\n");

        fwrite(pImage,1,768*576*2,file);
        fclose(file);
#endif
    }
    info.stride = bmi.biWidth * 3;// default RGB 8,8,8,
    if( filetype & MID811_FILE_JPEG)
    {
        image_to_jpg( pImage, szFilename, &info, qual);
    }
    if( filetype & MID811_FILE_BMP)
    {
        save_image_uncompressed( pImage, szFilename, &info, 0);
    }
    if( filetype & MID811_FILE_PPM)
    { 
        save_image_uncompressed( pImage, szFilename, &info, 1);
    }   

    // pBuffer must be non-null, or we'd never get here
    free( pBuffer);
    pBuffer = NULL;

    // free working buffers (pBuffer and possibly workImage if YUY2)
    if( pWorkImage) {
        free( pWorkImage);
        pWorkImage = NULL;
    }
    free( pBuffer);
    pBuffer = NULL;
    pImage = NULL;
    return 0;
}

MID811_API int __stdcall S811_SnapshotToFile( const char *szFilename, int filetype, int qual, int devid)
{
    LPWSTR pFilename;
    int hr;
    USES_CONVERSION;
    pFilename = A2OLE(szFilename);
    hr = S811_SnapshotToFileW(pFilename, filetype, qual, devid);
    return hr;
}

/** Test whether device was removed or not
* When  S811_SetRemoveMsg is called, a message window handle and remove message is set.
* When  remove message is received by the application, this function should be called
* to see if the device  was removed.  DLL should be shut down if device terminated
* @return 0 if  not removed, 1 if removed
*/
MID811_API int __stdcall S811_TestDeviceRemoval(int devid)
{
    S811_PREAMBLE(devid);
    long evCode;
    LONG_PTR param1, param2;
    if( theDLL.m_pEventDevSurpriseRemoval == NULL)
        return 0;
    while (SUCCEEDED(theDLL.m_pEventDevSurpriseRemoval->GetEvent(&evCode, &param1, &param2, 0)))
    {
        theDLL.m_pEventDevSurpriseRemoval->FreeEventParams(evCode, param1, param2);
        if (evCode == EC_DEVICE_LOST)
        {
            return 1;
        }
        break; 
    } 
    return 0; // device not removed
}


/** SetRecordMode encapsulates the SetRecord functions.  It is the preferred method of setting
*   the parameters.  In which case, the individual functions S811_SetRecordVideoStream, S811_SetRecordAudioStream
*   and S811_SetRecordAudioEncode are NOT required.
*   
*   @param recMode MID811_REC setting (see mid811types.h)
*   @return 0 on success
*/
MID811_API int __stdcall S811_SetRecordMode(MID811_REC rec, int devid)
{
    S811_PREAMBLE(devid);

    switch( rec)
    {
    case MID811_REC_MUX:
        S811_SetRecordAudioStream( TRUE, devid);
        S811_SetRecordVideoStream( TRUE, devid);
        break;
    case MID811_REC_PES:
        S811_SetRecordVideoStream( TRUE, devid);
        S811_SetRecordAudioStream( FALSE, devid);
        break;
    default:
        return -1;
    }
    theDLL.m_param[devid].m_recmode = rec;

    return 0;
}   

/** get the recording mode
 *  @param recMode MID811_REC setting (see mid811types.h)
 *  @return 0 on success
 */
MID811_API int __stdcall S811_GetRecordMode(MID811_REC *recMode, int devid)
{
    S811_PREAMBLE(devid);
    *recMode = theDLL.m_param[devid].m_recmode;
    return 0;
}

#if 0 
/** set audio input channel
*   @param channel audio input line-in, MID811_AUDIO_LINE, MID811_AUDIO_MIC
*   @param micBoost boolean, 1--boost 20dB, 0--no boost
*   @return 0 on success
*/
// 811, only one audio input for now
int CMid811Dll::SetAudioInput(MID811_AUDIO_INPUT audio, int micBoost, int devid)
{
    // TBD
    assert( m_pDC[devid] != NULL);
    return S_OK;
}

// not on 811
/** set audio input channel
*   @param channel audio input line-in, MID811_AUDIO_LINE, MID811_AUDIO_MIC
*   @param micBoost boolean, 1--boost 20dB, 0--no boost
*   @return 0 on success
*/
MID811_API int __stdcall S811_SetAudioInput(MID811_AUDIO_INPUT audio, int micBoost, int devid)
{
    HRESULT hr;
    S811_PREAMBLE(devid);

    hr = theDLL.SetAudioInput( audio, micBoost, devid);
    return hr;
}
#endif

MID811_API int __stdcall S811_GetBestVidRect( RECT *pRect, int devid)
{
    // get from 
    HRESULT hr;
    S811_PREAMBLE(devid);
    hr = theDLL.m_pDC[devid]->GetNativeVidSize( pRect);
    return hr;
}


MID811_API int __stdcall S811_GetVidSize(int *iVidSize, int devid)
{
    S811_PREAMBLE(devid);
    *iVidSize = theDLL.m_param[devid].m_iVidSize;
    return 0;
}

MID811_API int __stdcall S811_SetVidSize(MID811_RES eVidSize, int devid)
{
    int hr = S_OK;
    S811_PREAMBLE(devid);

    if( theDLL.m_param[devid].m_iVidSize != (int) eVidSize)
    {
        if( theDLL.m_pDC[devid]->IsGraphRunning()) 
        {
            return -1;
        }

        theDLL.m_param[devid].m_iVidSize = (int ) eVidSize;

    }
    return hr;
}


MID811_API int __stdcall S811_GetVideoSize( RECT *rect, int devid)
{
    S811_PREAMBLE(devid);
    return theDLL.m_pDC[devid]->GetNativeVidSize( rect);
}



MID811_API int __stdcall S811_EnableSnapshots( BOOL bOnOff, int devid)
{

    S811_PREAMBLE(devid);
    theDLL.m_pDC[devid]->m_bSnapshotEnable = bOnOff;
    return 0;
}



MID811_API int __stdcall S811_SetColorSpace(MID811_COLORSPACE mode, int devid)
{
    HRESULT hr = S_OK;
    S811_PREAMBLE( devid);
    theDLL.m_pDC[devid]->SetColorSpace(mode);
    return hr;
}



/** builds a direct show graph that connects the 811 only to sample grabber for video.
 *  For maximum capture efficiency( no preview or other directshow filters)
 *  mode == 1, Sample Grabber, YUY2
 *  mode == 0, Sample Grabber, RGB24
 * 
 */
MID811_API int __stdcall S811_StartCaptureOnly(int mode, int devid) 
{ 
    HRESULT hr = S_OK;
    S811_PREAMBLE( devid);

    hr = theDLL.m_pDC[devid]->BuildCaptureOnlySG( mode);
    if( FAILED(hr)) {
        return hr;
    }
    hr = theDLL.m_pDC[devid]->StartGraph();
    return hr;
}

MID811_API int __stdcall S811_StopCaptureOnly(int devid) 
{ 
    HRESULT hr = S_OK;
    S811_PREAMBLE( devid);
    theDLL.m_pDC[devid]->StopGraph();
    theDLL.m_pDC[devid]->CleanUp();

    return hr;
}


MID811_API int __stdcall S811_RegisterVidCB( cbfunc_t pfunc, int devid)
{
    S811_PREAMBLE( devid);
    theDLL.m_pDC[devid]->m_vidCB.SetCallback( pfunc);
    
    return 0;
}

MID811_API int __stdcall S811_RegisterAudCB( cbfunc_t pfunc, int devid)
{
    S811_PREAMBLE( devid);
    theDLL.m_pDC[devid]->m_audCB.SetCallback( pfunc);
    
    return 0;
}

MID811_API int __stdcall S811_StartCallback(int colorspace, int devid) 
{ 
    HRESULT hr = S_OK;
    S811_PREAMBLE( devid);

    hr = theDLL.m_pDC[devid]->BuildCaptureOnlySG_CB(colorspace);
    if( FAILED(hr)) {
        return hr;
    }
    hr = theDLL.m_pDC[devid]->StartGraph();
    return hr;
}

MID811_API int __stdcall S811_StopCallback(int devid) 
{ 
    HRESULT hr = S_OK;
    S811_PREAMBLE( devid);
    theDLL.m_pDC[devid]->StopGraph();
    theDLL.m_pDC[devid]->CleanUp();
    return hr;
}

