// mid811.h : main header file for the MID811 DLL
// Middle ware class.  Also see mid811func.h for exported functions from the DLL

#if !defined(AFX_MID811_H__54902B83_INCLUDE)
#define AFX_MID811_H__54902B83_INCLUDE

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000



#include "resource.h"		// main symbols


// forward declaration
class CMid811Dll;

#include "sensoraydevice.h"
#include <comdef.h>
#include "dshowhelper.h"
//#include "playback.h"
#include "interfaces.h"

//#include "playback.h"

#ifdef M2250_COMPAT
#include "PreviewManager.h"
#include "CaptureManager.h"
#endif

#include <vector>
using namespace std;

#define SAFE_DELETE(p)       { if(p) { delete (p);     (p)=NULL; } }



#define MAX_BACKGROUNDS 8
/////////////////////////////////////////////////////////////////////////////
// CMid811Dll
// See mid811.cpp for the implementation of this class




typedef struct
{
	// input source (video)
    MID811_SOURCE  m_input;
	// video system
	MID811_VIDSYS	m_vidsys;
	int				m_iVidSize;
  	MID811_ASPECT_MODE	m_AspectRatio;	// 0--stretch, 1--maintain with letter box
    char                m_bright;
    char                m_vsat;
    char                m_hue;
    char                m_contrast;
    MID811_REC          m_recmode;
    int                 m_fps;
    BOOL                m_bLOWLatency;
} user_board_params_t;



// Middle ware class.  Also see mid811func.h for exported functions
class CMid811Dll
{
    friend class CSensorayDevice;
public:
	CMid811Dll();
    ~CMid811Dll();

    int  FindNumDevices();
    void CleanUp();

	// set audio input
//	int SetAudioInput(MID811_AUDIO_INPUT audio, int micBoost, int devid);
	// set source or input channel
	int SetSource(MID811_SOURCE input, int devid );
	// Main High level Modules
    CSensorayDevice   *m_pDC[MAX_DEVICES];        // WIS device control
    // all user configurable parameters
    user_board_params_t  m_param[MAX_DEVICES];
	// surprise removal notification event
	IMediaEventExPtr m_pEventDevSurpriseRemoval;  
	BOOL			m_bInit;        // has been initialized
    // dll level
    BOOL            m_bOpen; // TRUE if DLL open, otherwise false
    int             m_iNumDevices;
private:
    // Helper function for SN_SnapshotToMem and SN_SnapshotToFile
    int SnapshotToMem( BYTE *image, int size, int freezetime, int wait, int *pRetW, int *pRetH);
	// test patterns on
    BOOL				m_bTest;  
};


#endif
