/* 811 device control. Copyright Sensoray Co. 2009 
*/
#ifndef SENSORAY_DEVICE_H_1234
#define SENSORAY_DEVICE_H_1234

#pragma once

#include "mid811func.h"
#include "mmreg.h"
#include "mid811.h"
#include "vmr9.h"
#include "interfaces.h"
#include "callback.h"

#define AVCAPTURE_ANALOG_VIDEO_INPIN_NAME	"Analog Video In"
#define AUDIO_PINNAME "Audio"
#define RAW_CAPTURE_PINNAME "Capture"

class CSensorayDevice
{
    CMid811Dll const *m_pDLL;
public:
	CSensorayDevice( const CMid811Dll *pDll);
	~CSensorayDevice(void);
    int           m_iInstance;

    HWND          m_hwnd;
    BOOL          m_bIsRecording;
    wchar_t      m_curFilePath[MAX_PATH];
	HRESULT InitDeviceControl();
    HRESULT SetOwnerWindow( HWND hwnd );
	HRESULT SetVideoPosition( RECT const rcVid, RECT const rcClip );
    HRESULT SetVcrMode(int mode);
    HRESULT SetVideoPin(int width, int height);
	HRESULT SetCrossbar(PhysicalConnectorType video_input);
    HRESULT SetCrossbarByPin(int pinnumber);
    HRESULT SetCrossbarAudioMute(BOOL muteval);
	HRESULT SetAnalogVideoDecoder(AnalogVideoStandard video_standard);
    HRESULT UpdateVideoSettings(BOOL preview, BOOL record );
    HRESULT UpdateVideoSettings();
    // set video input
    HRESULT SetInput(int input);

	HRESULT BuildPreviewGraph();
    HRESULT BuildRecordGraph(const wchar_t *file);
    
    HRESULT ConfigOldRenderer( IBaseFilterPtr VideoRenderer);
    HRESULT ConfigVMR7( IBaseFilterPtr VideoRenderer);
    HRESULT ConfigVMR9( IBaseFilterPtr VideoRenderer);

    HRESULT StartGraph		 (); 
    HRESULT PauseGraph		 (); 
    HRESULT ResumeGraph		 (); 
	void    StopGraph();
    void    CleanUp();
	BOOL IsGraphRunning() const;
	void SetRecordVideo( BOOL bRec);
	BOOL GetRecordVideo( ) const;
	void SetRecordAudio( BOOL bRec);
	BOOL GetRecordAudio( ) const;
	void SetMute( BOOL bMute);
	BOOL GetMute() const;
	void SetRenderVideo( BOOL bRenderVid);
	BOOL GetRenderVideo( ) const;

    HRESULT SetBrightness(int brightness);
    HRESULT SetContrast(int contrast);
    HRESULT SetHue(int hue);
    HRESULT SetSaturation(int saturation);
    BOOL    m_bVMR9;

public:
    HRESULT GetSnapshotBuffer( long *lSize, void **pBuffer, BITMAPINFOHEADER *pBMI = NULL );
    HRESULT GetSnapshotBufferStatic( long *lSize, unsigned char *pBuffer,  int type, BITMAPINFOHEADER *pBMI );
    HRESULT GetSnapshotBufferNC( long *lSize, void **pBuffer, BITMAPINFOHEADER *pBMI = NULL );
    HRESULT GetSnapshotBufferStaticNC( long *lSize, unsigned char *pBuffer, BITMAPINFOHEADER *pBMI );
public:
    BOOL GetDevicePresent() { return m_bDevicePresent;}
    HRESULT GetNativeVidSize( RECT *rect);
    HRESULT RepaintVideo(HDC hdc);
    HRESULT SetAspectRatioMode(MID811_ASPECT_MODE mode);
    void SetColorSpace(MID811_COLORSPACE mode) { m_colorspace = mode;};

private:
    BOOL m_bRAWSnapshots;
    BOOL m_bNC;      // 811 no-compression
    int  m_iNumSnap; // number of captures
    BITMAPINFOHEADER m_bmiSG; // 
    MID811_COLORSPACE m_colorspace;  // 
	RECT m_rcVid;
	RECT m_rcClip;
    BOOL m_bDevicePresent;
    void ShutDown();
   	void QueryInterfaces();//IAdvanced, IAccessFunc, IOSD
	void ReleaseInterfaces();
public:
    HRESULT BuildCaptureOnlySG(int mode);  // buffering SG
    HRESULT BuildCaptureOnlySG_CB(int colorspace);  // callback SG (see console demo)

private:
    ISampleGrabberPtr           m_pISG;
    HRESULT                     BuildSourceGraph();
    HRESULT Build811PreviewGraph();
    void SetPSFileName(CString FileName);
    IAMStreamConfigPtr        m_pAudioCaptureStreamConfig;
	IAMCrossbarPtr            m_pXBar;
    IAMVideoProcAmpPtr        m_pIVidProcAmp; // 7113, 7115 PVX322 video decoder or sensor 
	IAMAnalogVideoDecoderPtr  m_pAnalogVideoDecoder;
protected:
    IBaseFilterPtr            m_pVideoRenderer;
public:
    IBaseFilterPtr            m_pDump;
    IGraphBuilderPtr          m_pGB;
    IBaseFilterPtr            m_p811Proxy;
    IBaseFilterPtr            m_pfFileWriter;
    IVMRWindowlessControlPtr  m_pWc7;   // pointer to windowless control	
    IVMRWindowlessControl9Ptr m_pWc9;   // pointer to windowless control DirectX 9
    HRESULT                 DisplayModeChanged();
private:
    CCritSec                m_critSec;
	BOOL                    m_bAudioHWPresent;
	BOOL m_bHasCrossbar;  // If crossbar exists
	BOOL m_bHasAudioPin;  // if audio pin exists.  should be true
    BOOL m_bRun;   // if main source graph running
    // all options below, if set, require the graph to be rebuilt.
	BOOL m_bRecordVideo;	// record video
	BOOL m_bRecordAudio;	// record audio
	BOOL m_bMute;			// mute audio
	BOOL m_bRenderVideo;	// render video?  Why?  In case user wants to record, but not view the video
public:
    BOOL m_bSnapshotEnable; // enables turning off snapshots(for performance).
	ISampleGrabberPtr           m_pISG_vid;
	ISampleGrabberPtr           m_pISG_aud;
	CCallback                   m_audCB;
	CCallback                   m_vidCB;
};

#endif /*DEVICE_CONTROL_H*/
