#ifndef MID811_CONST_H
#define MID811_CONST_H


#ifdef ISDLLBUILD
    #define MID811_API extern "C" __declspec(dllexport)
#else
#ifdef __cplusplus
    #define MID811_API extern "C" __declspec(dllimport)
#else
    #define MID811_API __declspec(dllimport)
#endif
#endif


// Note: not all video cards will support all colorspaces
typedef enum
{
	MID811_COLORSPACE_YUY2, // default 
    MID811_COLORSPACE_RGB32,
    MID811_COLORSPACE_RGB24,
    MID811_COLORSPACE_UYVY,
}  MID811_COLORSPACE;




/** Video input source.  Note: Changing input may stop MPEG Video Stream
 * @param MID811_SOURCE_COMPOSITE_0 Composite channel 0
 * @param MID811_SOURCE_SVIDEO_0 S-Video Channel 0
 * @param MID811_SOURCE_NUM maximum number of inputs
 */
typedef enum
{
	MID811_SOURCE_COMPOSITE_0 = 1,
    MID811_SOURCE_SVIDEO_0 = 2,
    MID811_SOURCE_NUM	   = 2    // maximum number of inputs
} MID811_SOURCE;

// error constants 
#define MID811_SUCCESS             0
#define MID811_ERR_NONE            0
#define MID811_ERR_UNSPECIFIED     -1
#define MID811_ERR_CAPTURING       -2   // already capturing/recording
#define MID811_ERR_NOTCAPTURING    -3   // not capturing/recording
#define MID811_ERR_RUNNING         -4   // attempt to set configuration while graph running
#define MID811_ERR_SNAPSHOT        -5   // currently displaying snapshot
#define MID811_ERR_INVALIDFILE     -6   // invalid file
#define MID811_ERR_DIRECTX         -7   // directX error( other specific directX errors are possible)
#define MID811_ERR_NODEVICE		-8   // device not present
#define MID811_ERR_NOTINITIALIZED  -9	 // initialization function failed or not called
#define MID811_ERR_INVALIDMODE    -10  // invalid mode(eg.  record in MJPEG CAPTURE mode)

typedef enum
{
	MID811_VIDSYS_PAL=1,
	MID811_VIDSYS_NTSC=2
} MID811_VIDSYS;




/** VIDEO levels */
#define MID811_LEVEL_CONTRAST		1
#define	MID811_LEVEL_BRIGHTNESS	    2
#define MID811_LEVEL_SATURATION	    4
#define MID811_LEVEL_HUE			8


// image file save bitmask types 
 
#define	MID811_FILE_JPEG		1
#define	MID811_FILE_BMP			2
#define	MID811_FILE_PPM			4

/** General Status information
 * @param bIsRecording true if currently recording
 * @param bIsPlaying true if video playing(for debug)      
 * @param iFilesize current recorded file size
 * @param szFilepath current filepath being recorded
 */
typedef struct 
{ 
    ULONG       iFilesize;              // current recorded size(megs)
    char        szFilepath[MAX_PATH];   // current filepath
    BOOL        bIsRecording;           // true if currently recording
    BOOL        bIsPlaying;             // true if video streaming(preview)
    BOOL        bRecordAudio;
    BOOL        bRecordVideo;
} MID811STATUS; 

// unicode version
typedef struct 
{ 
    ULONG       iFilesize;              // current recorded size(megs)
    wchar_t     szFilepath[MAX_PATH];   // current filepath
    BOOL        bIsRecording;           // true if currently recording
    BOOL        bIsPlaying;             // true if video streaming(preview)
    BOOL        bRecordAudio;
    BOOL        bRecordVideo;
} MID811STATUS_W; 



/** recording mode for saved MPEG streams
 *  @param MID811_REC_MUX AVI
 *  @param MID811_REC_PES video only AVI
 */
typedef enum
{
	MID811_REC_MUX = 0,  // default: multiplexed(video+audio)
	MID811_REC_PES = 1   // video only AVI
} MID811_REC;

typedef enum
{	
	MID811_ASPECT_NONE = 0,	// no aspect ratio change
	MID811_ASPECT_CONST = 1	// maintain aspect ratio
} MID811_ASPECT_MODE;


/** Resolutions
 *  D1  -- 720x 480 NTSC, 720x576 PAL
 *  SIF -- 352 x 240  // NTSC only
 *  CIF -- 352 x 288  // PAL only
 *  VGA -- 640x480
 *  4SIF -- 704x480 // NTSC only
 *  4CIF -- 704x576 // PAL only
 *  QCIF -- 176x144 // 
 */
typedef enum
{
	MID811_RES_D1 = 0,
	MID811_RES_SIF,  // NTSC only (except 811NC)
    MID811_RES_CIF,  // PAL only  (except 811NC)
    MID811_RES_VGA,
    MID811_RES_QCIF,
    MID811_RES_QVGA, // 811 NC only
    MID811_RES_4SIF, // 811 NC only
    MID811_RES_4CIF, // 811 NC only
    MID811_RES_D5,  // 811 NC only
    MID811_RES_2SIF, // 811 NC only
    MID811_RES_QQVGA, // 811 NC only
    MID811_RES_SQCIF, //// 811 NC only
    MID811_RES_D1N, //// 811 NC only
    MID811_RES_D1P, //// 811 NC only
}  MID811_RES;



#endif
