/** midplayback API.
 * Copyright 2006-2009 Sensoray Company Inc.
 */
// Playback functions
// The Sensoray devices function without this APIs.  It is not a requirement.
// Provided as is.
// The SDK developer is responsible for writing their own playback features if more functionality is
// required.

#ifndef _MIDPLAYBACKFUNC_H
#define _MIDPLAYBACKFUNC_H

// error constants 
#define MIDPLAYBACK_SUCCESS             0
#define MIDPLAYBACK_ERR_NONE            0
#define MIDPLAYBACK_ERR_UNSPECIFIED     -1
#define MIDPLAYBACK_ERR_CAPTURING       -2   // already capturing/recording
#define MIDPLAYBACK_ERR_NOTCAPTURING    -3   // not capturing/recording
#define MIDPLAYBACK_ERR_RUNNING         -4   // attempt to set configuration while graph running
#define MIDPLAYBACK_ERR_SNAPSHOT        -5   // currently displaying snapshot
#define MIDPLAYBACK_ERR_INVALIDFILE     -6   // invalid file
#define MIDPLAYBACK_ERR_DIRECTX         -7   // directX error( other specific directX errors are possible)
#define MIDPLAYBACK_ERR_NODEVICE		-8   // device not present
#define MIDPLAYBACK_ERR_NOTINITIALIZED  -9	 // initialization function failed or not called
#define MIDPLAYBACK_ERR_INVALIDMODE    -10  // invalid mode(eg.  record in MJPEG CAPTURE mode)


#ifdef ISDLLBUILD
    #define MIDPLAYBACK_API extern "C" __declspec(dllexport)
#else
#ifdef __cplusplus
    #define MIDPLAYBACK_API extern "C" __declspec(dllimport)
#else
    #define MIDPLAYBACK_API __declspec(dllimport)
#endif
#endif
/** Set the window for the video playback
 *  @param hwnd pointer to video display window
 *  @param reserved unused
 */

MIDPLAYBACK_API int __stdcall Sray_SetPlaybackWindow(HWND hwnd);



/** plays back the specified video clip in the current window
  * default is first 
  */
MIDPLAYBACK_API int __stdcall Sray_PlaybackVideo(const char *filename_plus_path );
// UNICODE version
MIDPLAYBACK_API int __stdcall Sray_PlaybackVideoW(const wchar_t *filename_plus_path );

/** stops video playback
  * @return 0 on success 
  */
MIDPLAYBACK_API int __stdcall Sray_StopPlayback( );

/** pause/resume playback 
  * @param bPause bPause = TRUE(pause), bPause=FALSE(resume) 
  * @return 0 on success
  */
MIDPLAYBACK_API int __stdcall Sray_PausePlayback( BOOL bPause );

MIDPLAYBACK_API int __stdcall Sray_GetPlaybackVideoSize( RECT *rect);


#endif
